# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  2014-09-29 Cornelius Kölbel, cornelius@privacyidea.org
#             Added saml tests
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''     
  Description:  functional tests
                
  Dependencies: -

'''
import time
import hmac
import logging
import binascii
import struct
import hashlib

from privacyidea.tests import TestController, url
from json import loads


import sys
(ma, mi, _, _, _,) = sys.version_info
pver = float(int(ma) + int(mi) * 0.1)


log = logging.getLogger(__name__)


class HmacOtp():
    def __init__(self, secret, counter=0, digits=6, hashfunc=hashlib.sha1):
        self.secret = secret
        self.counter = counter
        self.digits = digits

        ' set up hashlib'
        ty = type(hashfunc).__name__
        if  ty == 'str' or ty == 'unicode':
            self.hashfunc = self._getHashlib(hashfunc)
        else:
            self.hashfunc = hashfunc

    def _getHashlib(self, hLibStr):

        if hLibStr is None:
            return hashlib.sha1

        hashlibStr = hLibStr.lower()

        if hashlibStr == "md5":
            return hashlib.md5
        elif hashlibStr == "sha1":
            return hashlib.sha1
        elif hashlibStr == "sha224":
            return hashlib.sha224
        elif hashlibStr == "sha256":
            return hashlib.sha256
        elif hashlibStr == "sha384":
            return hashlib.sha384
        elif hashlibStr == "sha512":
            return hashlib.sha512
        else:
            return hashlib.sha1



    def calcHmac(self, counter=None):
        #log.error("hmacSecret()")
        counter = counter or self.counter

        ## retrieve the unicode key
        akey = self.secret

        #log.debug("[hmac] key: %s", akey)

        ## and convert it to binary        from privacyidea.lib.crypto import SecretObj
        key = binascii.unhexlify(akey)
        msg = struct.pack(">Q", counter)
        dige = hmac.new(key, msg, self.hashfunc)

        digStr = str(dige.digest())

        del akey
        del key
        del dige

        return digStr

    def truncate(self, digest):
        offset = ord(digest[-1:]) & 0x0f

        binary = (ord(digest[offset + 0]) & 0x7f) << 24
        binary |= (ord(digest[offset + 1]) & 0xff) << 16
        binary |= (ord(digest[offset + 2]) & 0xff) << 8
        binary |= (ord(digest[offset + 3]) & 0xff)

        return binary % (10 ** self.digits)


    def generate(self, counter=None):
        counter = counter or self.counter
        myHmac = self.calcHmac(counter)
        otp = unicode(self.truncate(myHmac))

        """  fill in the leading zeros  """
        sotp = (self.digits - len(otp)) * "0" + otp
        #log.debug("[generate] %s %s %s" % (str(counter), str(otp), str(sotp) ) )
        self.counter = counter + 1
        return sotp


class TestValidateController(TestController):
    """
    test the validate controller

    remark:
        validate test for the sms token test are in the
            test_sms2 and
            test_httpsms and
            test_challenge_response
    """

    def createMOtpToken(self):
        parameters = {
                      "serial"  : "M722362",
                      "type"    : "motp",
                      "otpkey"  : "1234567890123456",
                      "otppin"  : "1234",
                      "user"    : "root",
                      "pin"     : "pin",
                      "description" : "TestToken1",
                      }

        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response



    def createTOtpToken(self, hashlib):
        '''
        // Seed for HMAC-SHA1 - 20 bytes
        String seed = "3132333435363738393031323334353637383930";
        // Seed for HMAC-SHA256 - 32 bytes
        String seed32 = "3132333435363738393031323334353637383930" +
        "313233343536373839303132";
        // Seed for HMAC-SHA512 - 64 bytes
        String seed64 = "3132333435363738393031323334353637383930" +
        "3132333435363738393031323334353637383930" +
        "3132333435363738393031323334353637383930" +
        "31323334";
        '''
        ##

        if (hashlib == "SHA512"):
            otpkey = "31323334353637383930313233343536373839303132333435363738393031323334353637383930313233343536373839303132333435363738393031323334"
        elif (hashlib == "SHA256"):
            otpkey = "3132333435363738393031323334353637383930313233343536373839303132"
        else:
            otpkey = "3132333435363738393031323334353637383930"
        parameters = {
                          "serial"  : "TOTP",
                          "type"    : "totp",
                          # 64 byte key
                          "otpkey"  : otpkey,
                          "otppin"  : "1234",
                          "user"    : "root",
                          "pin"     : "pin",
                          "otplen"  : 8,
                          "description" : "time based HMAC TestToken1",
                          "hashlib": hashlib,
                          }


        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response

        try:
            hmac = HmacOtp(otpkey, digits=8, hashfunc=hashlib)
        except Exception as e:
            raise e

        return hmac


    def createTOtpValue(self, hmac, T0=None, shift=0, timeStepping=30):
        ret = ""
        try:
            if T0 is None:
                T0 = time.time() - shift
            counter = int((T0 / timeStepping) + 0.5)
            ret = hmac.generate(counter)

        except Exception as e:
            raise e

        return ret

    def createToken1(self):
        """
            otp[0]: 870581 :
            otp[1]: 793334 :
            otp[2]: 088491 :
            otp[3]: 013126 :
            otp[4]: 818771 :
            otp[5]: 454594 :
            otp[6]: 217219 :
            otp[7]: 250710 :
            otp[8]: 478893 :
            otp[9]: 517407 :
        """
        parameters = {
                      "serial": "F722362",
                      "otpkey" : "AD8EABE235FC57C815B26CEF3709075580B44738",
                      "user" : "root",
                      "pin": "pin",
                      "description" : "TestToken1",
                      }

        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response

    def createRealmToken1(self, realm):
        """
            otp[0]: 870581 :
            otp[1]: 793334 :
            otp[2]: 088491 :
            otp[3]: 013126 :
            otp[4]: 818771 :
            otp[5]: 454594 :
            otp[6]: 217219 :
            otp[7]: 250710 :
            otp[8]: 478893 :
            otp[9]: 517407 :
        """
        parameters = {
                      "serial"  : "F722362",
                      "otpkey"  : "AD8EABE235FC57C815B26CEF3709075580B44738",
                      "user"    : "root",
                      "pin"     : "pin",
                      "description" : "TestToken1",
                      }
        if realm is not None:
            parameters.update(realm)
        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response

    def createToken(self):
        serials = []
        parameters = {
                      "serial": "F722362",
                      "otpkey" : "AD8EABE235FC57C815B26CEF3709075580B44738",
                      "user" : "root",
                      "pin": "pin",
                      "description" : "TestToken1",
                      }

        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response

        serials.append(parameters.get('serial'))


        parameters = {
                      "serial": "F722363",
                      "otpkey" : "AD8EABE235FC57C815B26CEF3709075580B4473880B44738",
                      "user" : "root",
                      "pin": "pin",
                      "description" : "TestToken2",
                      }

        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response

        serials.append(parameters.get('serial'))

        ## test the update
        parameters = {
                      "serial": "F722364",
                      "otpkey" : "AD8EABE235FC57C815B26CEF37090755",
                      "user" : "root",
                      "pin": "Pin3",
                      "description" : "TestToken3",
                      }

        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response

        serials.append(parameters.get('serial'))

        parameters = {
                      "serial": "F722364",
                      "otpkey" : "AD8EABE235FC57C815B26CEF37090755",
                      "user" : "root",
                      "pin": "pin",
                      "description" : "TestToken3",
                      }

        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response

        serials.append(parameters.get('serial'))

        return serials

    def createToken2(self):
        parameters = {
                      "serial": "T2",
                      "otpkey" : "AD8EABE235FC57C815B26CEF3709075580B44738",
                      "user" : "root",
                      "pin": "T2PIN",
                      "description" : "TestToken2",
                      }

        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response

    def createToken3(self):
        parameters = {
                      "serial": "T3",
                      "otpkey" : "AD8EABE235FC57C815B26CEF3709075580B44738",
                      "user" : "root",
                      "pin": "T2PIN",
                      "description" : "TestToken3",
                      }

        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response

    def createTokenSMS(self):
        parameters = {
                      "serial": "SM1",
                      "user" : "root",
                      "pin": "test",
                      "description" : "TestSMS",
                      "type":"sms",
                      "phone":"007"
                      }

        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response


    def createSpassToken(self, serial=None):
        if serial is None:
            serial = "TSpass"
        parameters = {
                      "serial"      : serial,
                      "otpkey"      : "AD8EABE235FC57C815B26CEF3709075580B44738",
                      "user"        : "root",
                      "pin"         : "pin",
                      "description" : "TestToken1",
                      "type"        : "spass"
                      }

        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response
        return serial

    def test_cryptedPin(self):

        realmsResp = self.app.get(url(controller='system', action='getDefaultRealm'), params=None)
        log.debug(realmsResp)
        #assert '"value": true' in response

        parameters = { "realm"      : "mydef", }
        defRealmsResp = self.app.get(url(controller='system', action='setDefaultRealm'), params=parameters)
        log.debug(defRealmsResp)
        #assert '"value": true' in response

        serial = self.createSpassToken("mySpass")
        self.removeTokenBySerial(serial)


    def removeTokenBySerial(self, serial):

        parameters = {
                      "serial": serial,
                      }

        response = self.app.get(url(controller='admin', action='remove'), params=parameters)
        return response
    """
        Use case:
            user:                 w.Token / wo.Token / unknown
            PassOnUserNotFound:   true / False / 'unset'
            Realm:                _default_  / myDomain

    """

    def checkFalse(self, realm):

        parameters = {"user": "root", "pass": "pin870581"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"user": "postgres", "pass": "pin"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        parameters = {"user": "postgres"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        parameters = {"user": "UnKnownUser"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response


    def checkFalse2(self, realm):

        parameters = {"user": "postgres"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        log.error("response %s\n", response)
        # Test response...
        assert '"value": true' in response

        parameters = {"user": "postgres", "pass": "pin"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"user": "UnKnownUser"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        parameters = {"user": "root", "pass": "pin088491"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"user": "root"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        assert '"value": false' in response

        return

    def checkFalse3(self, realm):

        parameters = {"user": "postgres"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        parameters = {"user": "postgres", "pass": "pin"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        parameters = {"user": "UnKnownUser"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"user": "root", "pass": "pin818771"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"user": "root"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        assert '"value": false' in response

    def checkTrue(self, realm):

        parameters = {"user": "postgres", "pass": "pin"}
        parameters.update(realm)
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"user": "postgres"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"user": "UnKnownUser"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"user": "root", "pass": "pin217219"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"user": "root"}
        parameters.update(realm)

        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        assert '"value": false' in response



        """
            otp[0]: 870581 :
            otp[1]: 793334 :
            otp[2]: 088491 :
            otp[3]: 013126 :
            otp[4]: 818771 :
            otp[5]: 454594 :
            otp[6]: 217219 :
            otp[7]: 250710 :
            otp[8]: 478893 :
            otp[9]: 517407 :
        """

    def test_autousercheck(self):
        '''
        testing PassOnUserNoToken and UserNotFound.
        '''
        realm = {}

        self.createToken1()

        response = self.app.get(url(controller='system', action='getRealms'))
        #log.error("response %s\n",response)
        # Test response...
        #assert '"value": true' in response
        parameters = {"username":"*"}
        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        log.error("response %s\n", response)

        self.checkFalse(realm)

        parameters = {"PassOnUserNoToken": "True"}
        response = self.app.get(url(controller='system', action='setConfig'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"setConfig PassOnUserNoToken:True": true' in response

        self.checkFalse2(realm)

        parameters = {"PassOnUserNoToken": "False"}
        response = self.app.get(url(controller='system', action='setConfig'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"setConfig PassOnUserNoToken:False": true' in response

        parameters = {"PassOnUserNotFound": "True"}
        response = self.app.get(url(controller='system', action='setConfig'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"setConfig PassOnUserNotFound:True": true' in response

        self.checkFalse3(realm)

        parameters = {"PassOnUserNoToken": "True"}
        response = self.app.get(url(controller='system', action='setConfig'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"setConfig PassOnUserNoToken:True": true' in response

        self.checkTrue(realm)

        parameters = {"key":"PassOnUserNotFound"}
        response = self.app.get(url(controller='system', action='delConfig'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"delConfig PassOnUserNotFound": true' in response


        parameters = {"key":"PassOnUserNoToken"}
        response = self.app.get(url(controller='system', action='delConfig'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"delConfig PassOnUserNoToken": true' in response


        self.removeTokenBySerial("F722362")


    def test_check(self):
        '''
        checking several different tokens /validate/check
        '''
        self.createToken()

        parameters = {"user": "root", "pass": "pin123456"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        # Test response...
        assert '"value": false' in response

        parameters = {"serial": "F722362"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.FailCount": 1' in response


        ## check all 3 tokens - the last one is it
        parameters = {"user": "root", "pass": "pin280395"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response


        parameters = {"serial": "F722364"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.Count": 1' in response
        assert '"privacyIDEA.FailCount": 0' in response


        parameters = {"serial": "F722362"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.FailCount": 1' in response


        ## check all 3 tokens - the last one is it
        parameters = {"pin": "TPIN", "serial" : "F722364"}
        response = self.app.get(url(controller='admin', action='set'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"set pin": 1' in response

        ## check all 3 tokens - the last one is it
        parameters = {"user": "root", "pass": "TPIN552629"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"serial":"F722364"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.Count": 4' in response
        assert '"privacyIDEA.FailCount": 0' in response

        for i in range(1, 20):
            ## check if otp could be reused
            parameters = {"user": "root", "pass": "TPIN552629"}
            response = self.app.get(url(controller='validate', action='check'), params=parameters)
            #log.error("response %s\n",response)
            # Test response...
            assert '"value": false' in response

        parameters = {"serial":"F722364"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.FailCount": 10' in response



        self.removeTokenBySerial("F722364")
        self.removeTokenBySerial("F722363")
        self.removeTokenBySerial("F722362")


    def test_resync(self):

        self.createToken2()

        parameters = {"serial" : "T2", "otp1" :"719818", "otp2" :"204809"}
        response = self.app.get(url(controller='admin', action='resync'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"serial":"T2"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.Count": 40' in response


        parameters = {"user": "root", "pass": "T2PIN204809"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        # 957690
        parameters = {"user": "root", "pass": "T2PIN957690"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"serial":"T2"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.Count": 41' in response


        self.removeTokenBySerial("T2")




    def test_resync2(self):
        '''
            test of resync with two similar tokens
        '''


        self.createToken2()
        self.createToken3()

        parameters = {"user": "root", "pass": "T2PIN204809"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response


        parameters = {"user" : "root", "otp1" :"719818", "otp2" :"204809"}
        response = self.app.get(url(controller='admin', action='resync'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"serial":"T2"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.Count": 40' in response

        parameters = {"serial":"T3"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        log.error("response %s\n", response)
        # Test response...
        assert '"privacyIDEA.Count": 40' in response

        parameters = {"serial":"T3", "pin" : "T3PIN"}
        response = self.app.get(url(controller='admin', action='set'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"set pin": 1' in response


        parameters = {"user": "root", "pass": "T2PIN204809"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response



        # 957690
        parameters = {"user": "root", "pass": "T2PIN957690"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"serial":"T2"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.Count": 41' in response


        self.removeTokenBySerial("T2")
        self.removeTokenBySerial("T3")

    def test_autoresync(self):
        '''
        auto resync:

        use case:
        - on the otp device the otp became out of sync as the user triggered
          the generation of otps to often. Now he will be able to automaticaly resync
          his token automatically by providing two consecutive otp's.

        test implementaion
        - switch the autosync: /system/set?autosync=true
        - do two consecutive otp validation requests

        no test:
        - disable autosync and same test should fail
        - test no consecutive otp's
        - test otp's out of sync window

        '''

        self.createToken2()

        ## test resync of token 2
        parameters = {"AutoResync": "true"}
        response = self.app.get(url(controller='system', action='setConfig'), params=parameters)
        #log.error("response %s\n",response)
        assert 'setConfig AutoResync:true": true' in response

        # 35
        parameters = {"user": "root", "pass": "T2PIN732866"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        if '"value": false' not in response:
            log.error("response %s\n", response)
        # Test response...
        assert '"value": false' in response

        # 36
        parameters = {"user": "root", "pass": "T2PIN920079"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response


        parameters = {"user": "root", "pass": "T2PIN732866"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        parameters = {"user": "root", "pass": "T2PIN957690"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        self.removeTokenBySerial("T2")

        ###############################################
        ## no test

        ## no consecutive otps
        self.createToken2()

        ## test resync of token 2
        parameters = {"AutoResync": "true"}
        response = self.app.get(url(controller='system', action='setConfig'), params=parameters)
        #log.error("response %s\n",response)
        assert 'setConfig AutoResync:true": true' in response

        # 35
        parameters = {"user": "root", "pass": "T2PIN732866"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        # 37
        parameters = {"user": "root", "pass": "T2PIN328973"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        self.removeTokenBySerial("T2")


        ###############################################
        ## no test

        ## now unset the autosync
        self.createToken2()

        ## test resync of token 2
        parameters = {"AutoResync": "false"}
        response = self.app.get(url(controller='system', action='setConfig'), params=parameters)
        #log.error("response %s\n",response)
        assert 'setConfig AutoResync:false": true' in response

        # 35
        parameters = {"user": "root", "pass": "T2PIN732866"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        # 36
        parameters = {"user": "root", "pass": "T2PIN920079"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response



    def test_checkMOtp(self):

        self.createMOtpToken()

        parameters = {"serial": "M722362"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.FailCount": 0' in response


        parameters = {"user": "root", "pass": "pin7215e7"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        parameters = {"serial": "M722362"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.FailCount": 1' in response


        """
            only in selfTest mode, it's allowed to set
            the start time for the mobile otp
        """

        parameters = {"user": "root", "pass": "pin7215e7", "init" : "126753360"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...

        if self.isSelfTest == True:
            assert '"value": true' in response
        else:
            log.error("\n-------------------------\n motp not tested for correctness \n"
                       "please enable 'privacyidea.selfTest = True' in your *.ini")
            assert '"value": false' in response

        self.removeTokenBySerial("M722362")


    def test_checkOTPAlgo(self):

        """
           The test token shared secret uses the ASCII string value
           "12345678901234567890".  With Time Step X = 30, and the Unix epoch as
           the initial value to count time steps, where T0 = 0, the TOTP
           algorithm will display the following values for specified modes and
           timestamps.

          +-------------+--------------+------------------+----------+--------+
          |  Time (sec) |   UTC Time   | Value of T (hex) |   TOTP   |  Mode  |
          +-------------+--------------+------------------+----------+--------+
          |      59     |  1970-01-01  | 0000000000000001 | 94287082 |  SHA1  |
          |             |   00:00:59   |                  |          |        |
          |  1111111109 |  2005-03-18  | 00000000023523EC | 07081804 |  SHA1  |
          |             |   00:00:59   |                  |          |        |
          |  1111111111 |  2005-03-18  | 00000000023523ED | 14050471 |  SHA1  |
          |             |   01:58:31   |                  |          |        |
          |  1234567890 |  2009-02-13  | 000000000273EF07 | 89005924 |  SHA1  |
          |             |   23:31:30   |                  |          |        |
          |  2000000000 |  2033-05-18  | 0000000003F940AA | 69279037 |  SHA1  |
          |             |   03:33:20   |                  |          |        |
          | 20000000000 |  2603-10-11  | 0000000027BC86AA | 65353130 |  SHA1  |
          |             |   11:33:20   |                  |          |        |


          |      59     |  1970-01-01  | 0000000000000001 | 46119246 | SHA256 |
          |             |   00:00:59   |                  |          |        |
          |  1111111109 |  2005-03-18  | 00000000023523EC | 68084774 | SHA256 |
          |             |   01:58:29   |                  |          |        |
          |  1111111111 |  2005-03-18  | 00000000023523ED | 67062674 | SHA256 |
          |             |   01:58:31   |                  |          |        |
          |  1234567890 |  2009-02-13  | 000000000273EF07 | 91819424 | SHA256 |
          |             |   23:31:30   |                  |          |        |
          |  2000000000 |  2033-05-18  | 0000000003F940AA | 90698825 | SHA256 |
          |             |   03:33:20   |                  |          |        |
          | 20000000000 |  2603-10-11  | 0000000027BC86AA | 77737706 | SHA256 |
          |             |   11:33:20   |                  |          |        |


          |      59     |  1970-01-01  | 0000000000000001 | 90693936 | SHA512 |
          |             |   01:58:29   |                  |          |        |
          |  1111111109 |  2005-03-18  | 00000000023523EC | 25091201 | SHA512 |
          |             |   01:58:29   |                  |          |        |
          |  1111111111 |  2005-03-18  | 00000000023523ED | 99943326 | SHA512 |
          |             |   01:58:31   |                  |          |        |
          |  1234567890 |  2009-02-13  | 000000000273EF07 | 93441116 | SHA512 |
          |             |   23:31:30   |                  |          |        |
          |  2000000000 |  2033-05-18  | 0000000003F940AA | 38618901 | SHA512 |
          |             |   03:33:20   |                  |          |        |
          | 20000000000 |  2603-10-11  | 0000000027BC86AA | 47863826 | SHA512 |
          |             |   11:33:20   |                  |          |        |
          +-------------+--------------+------------------+----------+--------+
        """

        testVector = {
                     'SHA1' : [(59, '94287082'),
                                 (1111111109, '07081804'),
                                 (1111111111, '14050471'),
                                 (1234567890, '89005924'),
                                 (2000000000, '69279037'),
                                 (20000000000, '65353130'),
                                 ],
                     'SHA256' : [(59, '46119246'),
                                 (1111111109, '68084774'),
                                 (1111111111, '67062674'),
                                 (1234567890, '91819424'),
                                 (2000000000, '90698825'),
                                 (20000000000, '77737706'),
                                 ],
                     'SHA512' : [(59, '90693936'),
                                 (1111111109, '25091201'),
                                 (1111111111, '99943326'),
                                 (1234567890, '93441116'),
                                 (2000000000, '38618901'),
                                 (20000000000, '47863826'),
                                 ],

                     }
        try:
            for hashAlgo in testVector.keys():

                totp = self.createTOtpToken(hashAlgo)

                arry = testVector.get(hashAlgo)
                for tupp in arry:
                    (T0, otp) = tupp
                    val = self.createTOtpValue(totp, T0)
                    assert otp == val
        except Exception as e:
            log.error("Error in TOTP algorithm!!")
            raise Exception(e)
        return

    def test_checkTOtp(self):

        self.createTOtpToken("SHA1")

        parameters = {"serial": "TOTP"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.FailCount": 0' in response


        parameters = {"user": "root", "pass": "pin12345678"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        parameters = {"serial": "TOTP"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        log.info("1 response /admin/hhow %s\n" % response)
        # Test response...
        assert '"privacyIDEA.FailCount": 1' in response


        """
            only in selfTest mode, it's allowed to set
            the start time for the mobile otp
            ..
           |      59     |  1970-01-01  | 0000000000000001 | 94287082 |  SHA1  |
             ..
        """

        parameters = {"user": "root", "pass": "pin94287082", "init" : "59"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        log.info("2 response /validate/check %s\n" % response)
        #log.error("response %s\n",response)
        # Test response...

        if self.isSelfTest == True:
            assert '"value": true' in response
        else:
            log.error("\n-------------------------\n motp not tested for correctness \n"
                       "please enable 'privacyidea.selfTest = True' in your *.ini")
            assert '"value": false' in response

        """ second test value
          |  1111111109 |  2005-03-18  | 00000000023523EC | 07081804 |  SHA1  |
        """
        parameters = {"user": "root", "pass": "pin07081804", "init" : "1111111109"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        log.info("3 response /validate/check %s\n" % response)
        #log.error("response %s\n",response)
        # Test response...

        if self.isSelfTest == True:
            assert '"value": true' in response
        else:
            log.error("\n-------------------------\n totp not tested for correctness \n"
                       "please enable 'privacyidea.selfTest = True' in your *.ini")
            assert '"value": false' in response

        parameters = {"user": "root", "pass": "pin89005924", "init" : "1234567890"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...

        if self.isSelfTest == True:
            assert '"value": true' in response
        else:
            log.error("\n-------------------------\n totp not tested for correctness \n"
                       "please enable 'privacyidea.selfTest = True' in your *.ini")
            assert '"value": false' in response

        self.removeTokenBySerial("TOTP")

        """
           |      59     |  1970-01-01  | 0000000000000001 | 46119246 | SHA256 |
           |             |   00:00:59   |                  |          |        |
           |      59     |  1970-01-01  | 0000000000000001 | 90693936 | SHA512 |

        """

        self.createTOtpToken("SHA256")

        parameters = {"user": "root", "pass": "pin46119246", "init" : "59" }
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        log.error("response %s\n", response)
        # Test response...

        if self.isSelfTest == True:
            assert '"value": true' in response
        else:
            log.error("\n-------------------------\n totp not tested for correctness \n"
                       "please enable 'privacyidea.selfTest = True' in your *.ini")
            assert '"value": false' in response


        self.removeTokenBySerial("TOTP")


        self.createTOtpToken("SHA512")

        parameters = {"user": "root", "pass": "pin90693936", "init" : "59" }
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        log.error("response %s\n", response)
        # Test response...

        if self.isSelfTest == True:
            assert '"value": true' in response
        else:
            log.error("\n-------------------------\n totp not tested for correctness \n"
                       "please enable 'privacyidea.selfTest = True' in your *.ini")
            assert '"value": false' in response


        self.removeTokenBySerial("TOTP")

    def test_totp_resync(self):
        #return

        try:
            response = self.removeTokenBySerial("TOTP")
        except Exception as e:
            log.debug("Token does not existed: %r" % e)

        totp = self.createTOtpToken("SHA1")


        parameters = {"serial": "TOTP"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        assert '"privacyIDEA.FailCount": 0' in response

        parameters = {"DefaultSyncWindow":"200", }
        response = self.app.get(url(controller='system', action='setDefault'), params=parameters)
        assert '"set DefaultSyncWindow": true' in response

        parameters = {"AutoResync": "true"}
        response = self.app.get(url(controller='system', action='setConfig'), params=parameters)
        assert 'setConfig AutoResync:true": true' in response

        parameters = {"user": "root", "pass": "pin12345678"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        assert '"value": false' in response

        parameters = {"serial": "TOTP"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        log.error("response %s\n", response)
        assert '"privacyIDEA.FailCount": 1' in response


        '''
            now test TOTP resync - backward lookup
            This test usese the verified HMAC algo
            for generating hmac keys
        '''



        myTime = time.time()

        otp1 = self.createTOtpValue(totp, myTime - 100)
        otp2 = self.createTOtpValue(totp, myTime - 70)

        parameters = {"user" : "root", "otp1" : otp1, "otp2" : otp2 }
        response = self.app.get(url(controller='admin', action='resync'), params=parameters)
        #assert '"value": true' in response



        '''
            now test TOTP resync - forward lookup
            This test usese the verified HMAC algo
            for generating hmac keys
        '''

        myTime = time.time()

        otp1 = self.createTOtpValue(totp, myTime + 122)
        otp2 = self.createTOtpValue(totp, myTime + 152)

        parameters = {"user" : "root", "otp1" : otp1, "otp2" : otp2 }
        response = self.app.get(url(controller='admin', action='resync'), params=parameters)
        assert '"value": true' in response

        self.removeTokenBySerial("TOTP")

    def test_totp_autosync(self):
        '''
            now let's test the autosync !!!
        '''

        parameters = {"DefaultSyncWindow":"200", }
        response = self.app.get(url(controller='system', action='setDefault'), params=parameters)
        assert '"set DefaultSyncWindow": true' in response

        parameters = {"AutoResync": "true"}
        response = self.app.get(url(controller='system', action='setConfig'), params=parameters)
        assert 'setConfig AutoResync:true": true' in response


        try:
            response = self.removeTokenBySerial("TOTP")
        except Exception as e:
            log.debug("Token does not existed: %r" % e)

        totp = self.createTOtpToken("SHA512")

        myTime = time.time()

        otp1 = self.createTOtpValue(totp, myTime + 255)
        otp2 = self.createTOtpValue(totp, myTime + 286)


        parameters = {"user": "root", "pass": "pin" + otp1 }
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        log.error("response %s\n", response)


        parameters = {"user": "root", "pass": "pin" + otp2 }
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        log.error("response %s\n", response)

        self.removeTokenBySerial("TOTP")



    def test_failCount(self):
        """
        Idea: test if MaxFailCount works and if Token could not be resetted in case of a
                valid OTP if MaxFailCount exceeded
        """


        self.createToken1()

        parameters = {"serial": "F722362", "MaxFailCount":"15"}
        response = self.app.get(url(controller='admin', action='set'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"set MaxFailCount": 1' in response

        parameters = {"user": "root", "pass": "pin870581"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response

        parameters = {"serial": "F722362"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.FailCount": 0' in response


        """
            Test if FailCount increments and in case of a valid OTP is resetted
        """
        for i in range(0, 14):
            parameters = {"user": "root", "pass": "pin123456"}
            response = self.app.get(url(controller='validate', action='check'), params=parameters)
            # Test response...
            assert '"value": false' in response

        parameters = {"serial": "F722362"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.FailCount": 14' in response


        ## check all 3 tokens - the last one is it
        parameters = {"user": "root", "pass": "pin818771"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": true' in response


        parameters = {"serial": "F722362"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.Count": 5' in response
        assert '"privacyIDEA.FailCount": 0' in response

        """
            Test if FailCount increments and in case of a maxFailCount
            could not be reseted by a valid OTP
        """
        for i in range(0, 15):
            parameters = {"user": "root", "pass": "pin123456"}
            response = self.app.get(url(controller='validate', action='check'), params=parameters)
            # Test response...
            assert '"value": false' in response

        parameters = {"serial":"F722362"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.Count": 5' in response
        assert '"privacyIDEA.FailCount": 15' in response

        """
            the reset by a valid OTP must fail and
            the OTP Count must be incremented anyway
        """
        parameters = {"user": "root", "pass": "pin250710"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"value": false' in response

        parameters = {"serial":"F722362"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.Count": 8' in response
        assert '"privacyIDEA.FailCount": 15' in response

        parameters = {"serial":"F722362"}
        response = self.app.get(url(controller='admin', action='reset'), params=parameters)
        log.error("response %s\n", response)
        assert '"value": 1' in response


        parameters = {"serial":"F722362"}
        response = self.app.get(url(controller='admin', action='show'), params=parameters)
        #log.error("response %s\n",response)
        # Test response...
        assert '"privacyIDEA.Count": 8' in response
        assert '"privacyIDEA.FailCount": 0' in response

        self.removeTokenBySerial("F722362")

    def test_samlcheck(self):
        """
        Test the /validate/samlcheck
        """
        parameters = {"serial": "saml0001",
                      "otpkey": "AD8EABE235FC57C815B26CEF3709075580B44738",
                      "user": "samluser",
                      "pin": "test",
                      "type": "spass"
                      }

        response = self.app.get(url(controller='admin', action='init'),
                                params=parameters)
        assert '"value": true' in response

        parameters = {"allowSamlAttributes": "True"}
        response = self.app.get(url(controller='system', action='setConfig'),
                                params=parameters)

        parameters = {"user": "samluser", "pass": "test"}
        response = self.app.get(url(controller='validate', action='samlcheck'),
                                params=parameters)
        print response
        # Test response...
        if '"auth": true' not in response:
            log.error(response)
        assert '"auth": true' in response
        assert '"username": "samluser"' in response
        assert '"surname": "Schenk",' in response
        assert '"mobile": " +4917011111",' in response
        assert '"phone": " 0897777",' in response
        assert '"givenname": "Heinz",' in response
        assert '"email": " h.schenk@witzig.com"' in response

        self.removeTokenBySerial("saml0001")

    def test_unicode(self):
        '''
        checking /validate/check with corrupted unicode
        '''
        serials = self.createToken()

        parameters = {"user": "root", "pass": u"\xC0"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)

        self.assertTrue('"value": false' in response or
                        '"status": false' in response, response)

        for serial in serials:
            self.removeTokenBySerial(serial)

        return

    def test_simple_check(self):
        '''
        Testing simplecheck
        '''
        response = self.app.get(url(controller='admin', action='init'),
                                    params={'type': 'spass',
                                            'user': 'root',
                                            'pin': 'topSecret',
                                            'serial': 'simple634'})
        print response
        assert '"status": true' in response

        response = self.app.get(url(controller='validate', action='simplecheck'),
                                params={'user': 'root',
                                        'pass': 'topSecret'})
        print response
        assert ':-)' in response

        response = self.app.get(url(controller='validate', action='simplecheck'),
                                params={'user': 'root',
                                        'pass': 'wrongPW'})
        print response
        assert ':-(' in response

        return




# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  * 2014-09-12 added Motp URL. Cornelius Kölbel
#
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
It generates the URL for smartphone apps like
google authenticator
oath token
'''

import binascii
import base64

import logging
log = logging.getLogger(__name__)

from privacyidea.lib.policy import PolicyClass
from privacyidea.lib.config import get_privacyIDEA_config
from pylons import request, config, tmpl_context as c
from urllib import quote
from privacyidea.lib.log import log_with


@log_with(log)
def create_motp_url(user, realm, key, serial=""):
    '''
    This creates the motp url as described at
    http://huseynov.com/index.php?post=motp-vs-google-authenticator-and-a-new-otp-app
    
    The format is:
    motp://SecureSite:alice@wonder.land?secret=JBSWY3DPEHPK3PXP
    '''
    # For Token2 the OTPKEY is hexencoded, not base32!
    otpkey = key
    
    Policy = PolicyClass(request, config, c,
                         get_privacyIDEA_config())
    label = Policy.get_tokenlabel(user, realm, serial)
    allowed_label_len = 20
    label = label[0:allowed_label_len]
    url_label = quote(label)
    
    return "motp://privacyidea:%s?secret=%s" % (url_label, otpkey)


@log_with(log)
def create_google_authenticator_url(user, realm, key, type="hmac", serial=""):
    '''
    This creates the google authenticator URL.
    This url may only be 119 characters long.
    Otherwise we qrcode.js can not create the qrcode.
    If the URL would be longer, we shorten the username
    
    We expect the key to be hexlified!
    '''
    # policy depends on some lib.util

    if "hmac" == type.lower():
        type = "hotp"

    key_bin = binascii.unhexlify(key)
    # also strip the padding =, as it will get problems with the google app.
    otpkey = base64.b32encode(key_bin).strip('=')

    #'url' : "otpauth://hotp/%s?secret=%s&counter=0" % ( user@realm, otpkey )
    base_len = len("otpauth://%s/?secret=%s&counter=0" % (type, otpkey))
    max_len = 119
    allowed_label_len = max_len - base_len
    log.debug("we have got %s characters left for the token label" % str(allowed_label_len))

    Policy = PolicyClass(request, config, c,
                         get_privacyIDEA_config())
    label = Policy.get_tokenlabel(user, realm, serial)
    label = label[0:allowed_label_len]

    url_label = quote(label)

    return "otpauth://%s/%s?secret=%s&counter=0" % (type, url_label, otpkey)

@log_with(log)
def create_oathtoken_url(user, realm, otpkey, type="hmac", serial=""):
    #'url' : 'oathtoken:///addToken?name='+serial +
    #                '&key='+otpkey+
    #                '&timeBased=false&counter=0&numDigites=6&lockdown=true',

    timebased = ""
    if "totp" == type.lower():
        timebased = "&timeBased=true"

    Policy = PolicyClass(request, config, c,
                         get_privacyIDEA_config())
    label = Policy.get_tokenlabel(user, realm, serial)
    url_label = quote(label)

    url = "oathtoken:///addToken?name=%s&lockdown=true&key=%s%s" % (
                                                                  url_label,
                                                                  otpkey,
                                                                  timebased
                                                                  )
    return url




# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''     
  Description:  functional tests
                
  Dependencies: -

'''

import logging
from privacyidea.tests import TestController, url

log = logging.getLogger(__name__)

class TestOrphanedTokens(TestController):


    def test_httperror(self):

        param = { 'otpkey': 'AD8EABE235FC57C815B26CEF3709075580B44738',
                  'user': 'root', 'pin':'pin', 'serial':'T2', 'type':'spass', 'resConf':'def'
                 }

        response = self.app.get(url(controller='admin', action='init'), params=param)
        assert '"status": false,' in response


        param = { 'otpkey': 'AD8EABE235FC57C815B26CEF3709075580B44738',
                  'user': 'root', 'pin':'pin', 'serial':'T2', 'type':'spass', 'resConf':'def', 'httperror':'400'
                 }
        try:
            response = self.app.get(url(controller='admin', action='init'), params=param)
        except Exception as e:
            httperror = e.args[0]
            assert "400 Bad Request" in httperror


        param = { 'otpkey': 'AD8EABE235FC57C815B26CEF3709075580B44738',
                  'user': 'root', 'pin':'pin', 'serial':'T2', 'type':'spass', 'resConf':'def', 'httperror':''
                 }
        try:
            response = self.app.get(url(controller='admin', action='init'), params=param)
        except Exception as e:
            httperror = e.args[0]
            assert "500 Internal Server Error" in httperror

        return





# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''     
  Description:  functional tests
                
  Dependencies: -

'''
from mock import patch
import smtplib
import re
import time

from privacyidea.tests import TestController, url


class TestEmailtokenController(TestController):

    pin = '1234'
    default_email_address = 'paul@example.com'
    patch_smtp = None
    mock_smtp_instance = None
    challenge_validity = 5
    token_serial = 'LSEM12345678'

    def setUp(self):
        TestController.setUp(self)
        parameters = {
            'EmailProvider': 'privacyidea.lib.emailprovider.SMTPEmailProvider',
            'EmailProviderConfig': '{ "SMTP_SERVER": "mail.example.com",\
                               "SMTP_USER": "secret_user",\
                               "SMTP_PASSWORD": "secret_pasword" }',
            'EmailChallengeValidityTime': self.challenge_validity,
            'EmailBlockingTimeout': 0
        }
        response = self.app.get(url(controller='system', action='setConfig'),
                                params=parameters)
        assert '"status": true' in response

        # Enroll token
        parameters = {
            'type': 'email',
            'serial': self.token_serial,
            'description': "E-mail token enrolled in functional tests",
            'email_address': self.default_email_address
        }
        response = self.app.get(url(controller='admin', action='init'),
                                params=parameters)
        assert '"value": true' in response

        parameters = {"serial": self.token_serial,
                      "user": "root",
                      "pin": self.pin}
        self.unassign(self.token_serial)
        response = self.app.get(url(controller='admin', action='assign'),
                                params=parameters)
        assert '"value": true' in response

        # Patch (replace) smtplib.SMTP class to prevent e-mails from being
        # sent out
        self.patch_smtp = patch('smtplib.SMTP', spec=smtplib.SMTP)
        mock_smtp_class = self.patch_smtp.start()
        self.mock_smtp_instance = mock_smtp_class.return_value
        self.mock_smtp_instance.sendmail.return_value = []

    def tearDown(self):
        TestController.tearDown(self)
        self.patch_smtp.stop()

    def test_default(self):
        """
        Test the default case: enroll, assign, send challenge, get successful response
        """
        response, otp = self._trigger_challenge()
        self._assert_email_sent(response)
        response = self.app.get(url(controller='validate', action='check'),
                                params={'user': 'root', 'pass': self.pin + otp})
        response_json = response.json
        self.assertTrue(response_json['result']['status'])
        self.assertTrue(response_json['result']['value'])


    def test_multiple_challenges(self):
        """
        Test with multiple challenges

        To do this we extend the challenge validity time and set a small blocking timeout.
        By waiting 5 seconds after every request we make sure a new e-mail is sent (and challenge
        created). In the end we send a response with one of the challenges (not the last one).
        """
        parameters = {
            'EmailChallengeValidityTime': 120,
            'EmailBlockingTimeout': 3
        }
        response = self.app.get(url(controller='system', action='setConfig'),
                                params=parameters)
        assert '"status": true' in response

        # trigger 1st challenge
        response, _ = self._trigger_challenge()
        self._assert_email_sent(response)
        time.sleep(5)
        # trigger 2nd challenge
        response, _ = self._trigger_challenge()
        self._assert_email_sent(response)
        time.sleep(5)
        # trigger 3rd challenge and store resulting information
        stored_response, stored_otp = self._trigger_challenge()
        self._assert_email_sent(response)
        time.sleep(5)
        # trigger 4th challenge
        response, _ = self._trigger_challenge()
        self._assert_email_sent(response)

        # Send the response with the stored values from the 3rd challenge
        transaction_id = stored_response['detail']['transactionid']
        # since we are sending the transactionid we only need the otp (without pin)
        response = self.app.get(url(controller='validate', action='check'),
                                params={'user': 'root', 'pass': stored_otp, 'transactionid': transaction_id})
        response = response.json
        self.assertTrue(response['result']['status'])
        self.assertTrue(response['result']['value'])

    def test_timeout(self):
        """
        Test that challenges timeout after 'EmailChallengeValidityTime'
        """
        response, otp = self._trigger_challenge()
        self._assert_email_sent(response)
        time.sleep(int(self.challenge_validity * 1.2))  # we wait 120% of the challenge timeout
        response = self.app.get(url(controller='validate', action='check'),
                                params={'user': 'root', 'pass': self.pin + otp})
        response = response.json
        self.assertTrue(response['result']['status'])
        self.assertFalse(response['result']['value'], "Challenge should have timed out")

    def test_blocking(self):
        """
        Test that no new e-mails are sent out during EmailBlockingTimeout
        """
        parameters = {
            'EmailBlockingTimeout': 3
        }
        response = self.app.get(url(controller='system', action='setConfig'),
                                params=parameters)
        assert '"status": true' in response

        # Trigger 1st challenge (that should send e-mail)
        response, _ = self._trigger_challenge()
        self._assert_email_sent(response)

        # Trigger 2nd challenge (should send no e-mail)
        response, _ = self._trigger_challenge()
        self.assertEqual("e-mail with otp already submitted", response['detail']['message'])

        time.sleep(5)  # wait for blocking timeout to pass

        # Trigger 3rd challenge (that should send e-mail)
        response, otp = self._trigger_challenge()
        self._assert_email_sent(response)

        response = self.app.get(url(controller='validate', action='check'),
                                params={'user': 'root', 'pass': self.pin + otp})
        response_json = response.json
        self.assertTrue(response_json['result']['status'])
        self.assertTrue(response_json['result']['value'])

        time.sleep(5)  # wait again to prevent problems with other tests

    def test_smtplib_exceptions(self):
        """
        Verify that SMTPRecipientsRefused exception is caught and no challenge is created.

        We assume that this works for other smtplib exceptions as well, because from privacyIDEAs point
        of view they behave in the same way.
        """
        # Get existing challenges (to verify later that no new ones were added)
        existing_challenges = {}
        try:
            response_string = self.app.get(url(controller='admin', action='checkstatus'),
                                           params={'user': 'root'})
            response = response_string.json
            existing_challenges = response['result']['value']['values'][self.token_serial]['challenges']
        except KeyError:
            pass  # No challenges exist for this token

        senderrs = {self.default_email_address:  (450, '4.1.8 <test@invalid.subdomain.privacyidea.de>: ' +
                                                'Sender address rejected: Domain not found')}
        # Trigger SMTPRecipientsRefused exception when sendmail is called
        self.mock_smtp_instance.sendmail.side_effect = smtplib.SMTPRecipientsRefused(senderrs)
        response_string = self.app.get(url(controller='validate', action='check'),
                                       params={'user': 'root', 'pass': self.pin})
        response = response_string.json
        expected_error = "error sending e-mail " + str(senderrs)
        self.assertEqual(expected_error, response['detail']['message'], "Error message does not match")

        # Get new challenges
        response_string = self.app.get(url(controller='admin', action='checkstatus'),
                                       params={'user': 'root'})
        response = response_string.json
        new_challenges = response['result']['value']['values'][self.token_serial]['challenges']

        # Verify that no challenge was created (the exception should have prevented it)
        self.assertTrue(existing_challenges == new_challenges,
                        "No new challenges should have been created.")

    def _trigger_challenge(self):
        """
        Triggers a challenge by doing validate/check with only the pin

        :return: tuple of the response and the otp value
        :rtype: (dict, string)
        """
        response = self.app.get(url(controller='validate', action='check'),
                                params={'user': 'root', 'pass': self.pin})
        self.assertTrue(self.mock_smtp_instance.sendmail.call_count >= 1,
                        "smtplib.SMTP.sendmail() should have been called at least once")
        call_args = self.mock_smtp_instance.sendmail.call_args
        ordered_args = call_args[0]
        email_from = ordered_args[0]
        email_to = ordered_args[1]
        message = ordered_args[2]
        self.assertEqual("privacyidea@example.com", email_from)
        self.assertEqual(self.default_email_address, email_to)

        matches = re.search('\d{6}', message)
        self.assertTrue(matches is not None)
        otp = matches.group(0)
        self.assertEqual(6, len(otp))
        return response.json, otp

    def _assert_email_sent(self, response):
        """
        Assert that the response contains information stating that the e-mail with the challenge
        has been sent.

        :param response: The response returned by validate/check
        :response type: dict
        """
        self.assertEqual("e-mail sent successfully", response['detail']['message'])
        self.assertTrue(response['result']['status'])
        self.assertFalse(response['result']['value'])



# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  LSE
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
  Description:  This file contains the definition of the SMS token class
  
  Dependencies: -

'''

import time
import datetime
import traceback

from privacyidea.lib.HMAC    import HmacOtp
from privacyidea.lib.util    import getParam
from privacyidea.lib.util    import required

from privacyidea.lib.validate import check_pin
from privacyidea.lib.validate import split_pin_otp

from privacyidea.lib.config  import getFromConfig

from privacyidea.lib.policy import PolicyClass
from pylons import request, config, tmpl_context as c
from privacyidea.lib.config import get_privacyIDEA_config
from privacyidea.lib.log import log_with
from privacyidea.smsprovider.SMSProvider import getSMSProviderClass

from json import loads

from gettext import gettext as _


from privacyidea.lib.tokens.hmactoken import HmacTokenClass

import logging
log = logging.getLogger(__name__)

keylen = {'sha1'    : 20,
          'sha256'  : 32,
          'sha512'  : 64,
          }



class SmsTokenClass(HmacTokenClass):
    '''
    implementation of the sms token class
    '''
    def __init__(self, aToken):
        HmacTokenClass.__init__(self, aToken)
        self.setType(u"sms")
        self.hKeyRequired = False

        # we support various hashlib methods, but only on create
        # which is effectively set in the update
        self.hashlibStr = getFromConfig("hotp.hashlib", "sha1")
        self.mode = ['challenge']
        self.Policy = PolicyClass(request, config, c,
                                  get_privacyIDEA_config())

    @classmethod
    def getClassType(cls):
        '''
        return the generic token class identifier
        '''
        return "sms"

    @classmethod
    def getClassPrefix(cls):
        return "LSSM"

    @classmethod
    def getClassInfo(cls, key=None, ret='all'):
        '''
        getClassInfo - returns all or a subtree of the token definition

        :param key: subsection identifier
        :type key: string

        :param ret: default return value, if nothing is found
        :type ret: user defined

        :return: subsection if key exists or user defined
        :rtype : s.o.

        '''

        res = {
               'type' : 'sms',
               'title' : _('SMS Token'),
               'description' :
                    _('sms challenge-response token - hmac event based'),
               'init'         : { 'title'  : {'html'      : 'smstoken.mako',
                                             'scope'     : 'enroll.title', },
                                  'page' : {'html'      : 'smstoken.mako',
                                            'scope'      : 'enroll', },
                                   },

               'config'         : {'title'  : {'html'      : 'smstoken.mako',
                                             'scope'     : 'config.title', },
                                   'page' : {'html'      : 'smstoken.mako',
                                            'scope'      : 'config', },
                                   },

               'selfservice'   :  { 'enroll' :
                                   {'title'  :
                                    { 'html'      : 'smstoken.mako',
                                      'scope'     : 'selfservice.title.enroll',
                                      },
                                    'page' :
                                    {'html'       : 'smstoken.mako',
                                     'scope'      : 'selfservice.enroll',
                                     },
                                    },
                                  },



        }

        if key is not None and res.has_key(key):
            ret = res.get(key)
        else:
            if ret == 'all':
                ret = res

        return ret

    @log_with(log)
    def update(self, param, reset_failcount=True):
        '''
        update - process initialization parameters

        :param param: dict of initialization parameters
        :type param: dict

        :return: nothing

        '''
        # specific - phone
        phone = getParam(param, "phone", required)
        self.setPhone(phone)

        ## in case of the sms token, only the server must know the otpkey
        ## thus if none is provided, we let create one (in the TokenClass)
        if not param.has_key('genkey') and not param.has_key('otpkey'):
            param['genkey'] = 1

        HmacTokenClass.update(self, param, reset_failcount)
        return

    @log_with(log)
    def is_challenge_response(self, passw, user, options=None, challenges=None):
        '''
        check, if the request contains the result of a challenge

        :param passw: password, which might be pin or pin+otp
        :param user: the requesting user
        :param options: dictionary of additional request parameters

        :return: returns true or false
        '''

        challenge_response = False

        if "state" in options or "transactionid" in options:
            challenge_response = True

        ## it as well might be a challenge response,
        ## if the passw is longer than the pin
        if challenge_response == False:
            (res, pin, otpval) = split_pin_otp(self, passw, user=user,
                                                            options=options)
            if res >= 0 :
                res = check_pin(self, pin, user=user, options=options)
                if res == True and len(otpval) > 0:
                    challenge_response = True

        return challenge_response

### challenge interfaces starts here
    @log_with(log)
    def is_challenge_request(self, passw, user, options=None):
        '''
        check, if the request would start a challenge

        - default: if the passw contains only the pin, this request would
        trigger a challenge

        - in this place as well the policy for a token is checked

        :param passw: password, which might be pin or pin+otp
        :param options: dictionary of additional request parameters

        :retrun: returns true or false
        '''

        request_is_valid = False
        ## do we need to call the
        #(res, pin, otpval) = split_pin_otp(self, passw, user, options=options)

        pin_match = check_pin(self, passw, user=user, options=options)
        if pin_match is True:
            request_is_valid = True

        return request_is_valid

    ##
    ##!!! this function is to be called in the sms controller !!!
    ##
    @log_with(log)
    def submitChallenge(self, options=None):
        '''
        submit the sms message - former method name was checkPin

        :param options: the request options context

        :return: tuple of success and message
        '''
        res = 0
        user = None
        message = "<otp>"

        ## it is configurable, if sms should be triggered by a valid pin
        send_by_PIN = getFromConfig("sms.sendByPin") or True

        if self.isActive() == True and send_by_PIN == True :
            counter = self.getOtpCount()
            log.debug("counter=%r" % counter)
            self.incOtpCounter(counter, reset=False)
            # At this point we must not bail out in case of an
            # Gateway error, since checkPIN is successful. A bail
            # out would cancel the checking of the other tokens
            try:

                if options is not None and type(options) == dict:
                    user = options.get('user', None)
                    if user:
                        _sms_ret, message = self.Policy.get_auth_smstext(
                                                realm=user.realm)
                res, message = self.sendSMS(message=message)
                self.info['info'] = "SMS sent: %r" % res

            except Exception as e:
                # The PIN was correct, but the SMS could not be sent.
                self.info['info'] = unicode(e)
                info = ("The PIN was correct, but the"
                          " SMS could not be sent: %r" % e)
                log.warning(info)
                res = False
                message = info
        if len(message) == 0:
            pass

        return res, message


    def initChallenge(self, transactionid, challenges=None, options=None):
        """
        initialize the challenge -
        in the privacyidea server a challenge object has been allocated and
        this method is called to confirm the need of a new challenge
        or if for the challenge request, there is an already outstanding
        challenge to which then could be referred (s. ticket #2986)

        :param transactionid: the id of the new challenge
        :param options: the request parameters

        :return: tuple of
                success - bool
                transid - the best transaction id for this request context
                message - which is shown to the user
                attributes - further info (dict) shown to the user
        """

        success = True
        transid = transactionid
        message = 'challenge init ok'
        attributes = {}

        now = datetime.datetime.now()
        blocking_time = int(getFromConfig('SMSBlockingTimeout', 60))

        for challenge in challenges:
            start = challenge.get('timestamp')
            expiry = start + datetime.timedelta(seconds=blocking_time)
            ## check if there is already a challenge underway
            if now >= start and now <= expiry:
                transid = challenge.getTransactionId()
                message = 'sms with otp already submitted'
                success = False
                attributes = {'info' : 'challenge already submitted',
                              'state' : transid}
                break

        return (success, transid, message, attributes)

    @log_with(log)
    def createChallenge(self, transactionid, options=None):
        """
        create a challenge, which is submitted to the user

        :param transactionid: the id of this challenge
        :param options: the request context parameters / data
        :return: tuple of (bool, message and data)
                 bool, if submit was successful
                 message is submitted to the user
                 data is preserved in the challenge
                 attributes - additional attributes, which are displayed in the
                    output
        """
        success = False
        sms = ""
        message = ""
        attributes = {'state':transactionid}

        success, sms = self.submitChallenge(options=options)

        if success is True:
            message = 'sms submitted'
            self.setValidUntil()
        else:
            attributes = {'state' : ''}
            message = 'sending sms failed'
            if len('sms') > 0:
                message = sms

        ## after submit set validity time in readable
        ## datetime format in the storeing data
        timeScope = self.loadprivacyIDEASMSValidTime()
        expiryDate = datetime.datetime.now() + \
                                    datetime.timedelta(seconds=timeScope)
        data = {'valid_until' : "%s" % expiryDate }

        return (success, message, data, attributes)

    @log_with(log)
    def checkResponse4Challenge(self, user, passw, options=None, challenges=None):
        """
        verify the response of a previous challenge

        :param user:     the requesting user
        :param passw:    the to be checked pass (pin+otp)
        :param options:  options an additional argument, which could be token
                          specific
        :param challenges: the list of challenges, where each challenge is
                            described as dict
        :return: tuple of (otpcounter and the list of matching challenges)

        do the standard check for the response of the challenge +
        change the tokeninfo data of the last challenge
        """
        otp_count = -1
        matching = []

        tok = super(SmsTokenClass, self)
        counter = self.getOtpCount()
        window = self.getOtpCountWindow()

        now = datetime.datetime.now()
        timeScope = self.loadprivacyIDEASMSValidTime()

        otp_val = passw

        ## fallback: do we have pin+otp ??
        (res, pin, otp) = split_pin_otp(self, passw, user=user,
                                                            options=options)

        if res >= 0:
            res = check_pin(self, pin, user=user, options=options)
            if res == True:
                otp_val = otp

        for challenge in challenges:
            otp_count = self.checkOtp(otp_val, counter, window,
                                                            options=options)
            if otp_count > 0 :
                matching.append(challenge)
                break

        return (otp_count, matching)

    @log_with(log)
    def checkOtp(self, anOtpVal, counter, window, options=None):
        '''
        checkOtp - check the otpval of a token against a given counter
        in the + window range

        :param passw: the to be verified passw/pin
        :type passw: string

        :return: counter if found, -1 if not found
        :rtype: int
        '''
        ret = HmacTokenClass.checkOtp(self, anOtpVal, counter, window)
        if ret != -1:
            if self.isValid() == False:
                ret = -1

        if ret >= 0:
            if self.Policy.get_auth_AutoSMSPolicy():
                user = None
                message = "<otp>"
                if options is not None and type(options) == dict:
                    user = options.get('user', None)
                    if user:
                        sms_ret, message = self.Policy.get_auth_smstext(realm=user.realm)
                self.incOtpCounter(ret, reset=False)
                success, message = self.sendSMS(message=message)

        if ret >= 0:
            msg = "otp verification was successful!"
        else:
            msg = "otp verification failed!"
        log.debug("end. %s ret: %r" % (msg, ret))
        return ret


    @log_with(log)
    def getNextOtp(self):
        '''
        access the next valid otp

        :return: otpval
        :rtype: string
        '''
        try:
            ### TODO - replace tokenLen
            otplen = int(self.token.privacyIDEAOtpLen)
        except ValueError as ex:
            log.error("ValueError %r" % ex)
            raise Exception(ex)

        secret_obj = self.token.getHOtpKey()
        counter = self.token.getOtpCounter()

        #log.debug("serial: %s",serialNum)
        hmac2otp = HmacOtp(secret_obj, counter, otplen)
        nextotp = hmac2otp.generate(counter + 1)

        return nextotp

    ### in the SMS token we use the generic TokenInfo
    ### to store the phone number
    def setPhone(self, phone):
        '''
        setter for the phone number

        :param phone: phone number
        :type phone:  string

        :return: nothing
        '''
        self.setSMSInfo("phone", phone)
        return

    def setUntil(self, until):
        '''
        This is the time the sent OTP value is valid/can be used.
                                                        (internal function)

        :param until: until time in unix time sec
        :type until:  int

        :return: nothing
        '''

        self.setSMSInfo("until", until)
        return

    def getPhone(self):
        '''
        getter for the phone number

        :return:  phone number
        :rtype:  string
        '''

        (phone, _till) = self.getSMSInfo()
        return phone

    def getUntil(self):
        '''
        getter for the until time definition

        :return:  until time definition of unix time sec
        :rtype:  int
        '''

        (_phone, until) = self.getSMSInfo()

        ## suport for direct verification
        if until == 0:
            timeScope = self.loadprivacyIDEASMSValidTime()
            until = int (time.time()) + timeScope
        return until

    @log_with(log)
    def setSMSInfo(self, key, value):
        '''
        generic method to set the sms infos like phone or validity in the
        tokeninfo (json) entry

        :param key: name of the hash key
        :type key:  string
        :param value: value of the entry
        :type value: any

        :return: nothing
        '''
        self.addToTokenInfo(key, value)
        return

    @log_with(log)
    def getSMSInfo(self):
        '''
        retrieve the phone number and the validity scope

        :return: tuple of phone number and validity time in unix lifetime sec
        '''

        phone = ""
        until = 0

        info = self.getTokenInfo()

        if info.has_key("phone") == True:
            phone = info.get("phone")

        if info.has_key("until") == True:
            until = info.get("until")

        return (phone, until)


    ### we take the countWindow.column to store the time
    ##  in int format (cut off the .2 from the time() )
    ###
    @log_with(log)
    def setValidUntil(self):
        '''
        adjust the timeframe of validity

        :return: nothing
        '''
        timeScope = self.loadprivacyIDEASMSValidTime()
        dueDate = int (time.time()) + timeScope
        self.setUntil(dueDate)
        #self.token.setCountWindow(dueDate)

        return dueDate

    @log_with(log)
    def isValid(self):
        '''
        check if sms challenge is still valid

        :return: True or False
        :rtype: boolean
        '''
        ret = False
        dueDate = self.getUntil()
        now = int(time.time())
        if dueDate >= now:
            ret = True
        if ret == True:
            msg = "the sms challenge is still valid"
        else:
            msg = "the sms challenge is no more valid"
        log.debug("%s: ret: %r" % (msg, ret))
        return ret

    @log_with(log)
    def sendSMS(self, message="<otp>"):
        '''
        send sms

        :param message: the sms submit message - could contain placeholders
         like <otp> or <serial>
        :type message: string

        :return: submitted message
        :rtype: string

        '''
        ret = None

        phone = self.getPhone()
        otp = self.getNextOtp()
        serial = self.getSerial()

        message = message.replace("<otp>", otp)
        message = message.replace("<serial>", serial)

        log.debug("sending SMS to phone number %s " % phone)
        (SMSProvider, SMSProviderClass) = self.loadprivacyIDEASMSProvider()
        log.debug("smsprovider: %s, class: %s"
                                             % (SMSProvider, SMSProviderClass))

        try:
            sms = getSMSProviderClass(SMSProvider, SMSProviderClass)()
        except Exception as exc:
            log.error("Failed to load SMSProvider: %r" % exc)
            log.error(traceback.format_exc())
            raise exc

        try:
            ## now we need the config from the env
            log.debug("loading SMS configuration for class %s" % sms)
            config = self.loadprivacyIDEASMSProviderConfig()
            log.debug("config: %r" % config)
            sms.loadConfig(config)
        except Exception as exc:
            log.error("Failed to load SMSProviderConfig: %r" % exc)
            log.error(traceback.format_exc())
            raise Exception("Failed to load SMSProviderConfig: %r" % exc)

        log.debug("submitMessage: %r, to phone %r" % (message, phone))
        ret = sms.submitMessage(phone, message)
        log.debug("message submitted")

        ## after submit set validity time
        self.setValidUntil()

        # return OTP for selftest purposes
        return ret, message

    @log_with(log)
    def loadprivacyIDEASMSProvider(self):
        '''
        get the SMS Provider class definition

        :return: tuple of SMSProvider and Provider Class as string
        :rtype: tuple of (string, string)
        '''
        smsProvider = getFromConfig("SMSProvider")

        if smsProvider is not None:
            (SMSProvider, SMSProviderClass) = smsProvider.rsplit(".", 1)

        return (SMSProvider, SMSProviderClass)

    @log_with(log)
    def loadprivacyIDEASMSProviderConfig(self):
        '''
        load the defined sms provider config definition

        :return: dict of the sms provider definition
        :rtype: dict
        '''
        ## get User realm
        config = {}

        tConfig = getFromConfig("encprivacyidea.SMSProviderConfig", None)
        if tConfig is None:
            tConfig = getFromConfig("SMSProviderConfig", "{}")

        ## fix the handling of multiline config entries
        lconfig = []
        lines = tConfig.splitlines()
        for line in lines:
            line = line.strip('\\')
            if len(line) > 0:
                lconfig.append(line)

        tConfig = " ".join(lconfig)
        log.debug("providerconfig: %s" % tConfig)

        if tConfig is not None:
            config = loads(tConfig)

        return config

    @log_with(log)
    def loadprivacyIDEASMSValidTime(self):
        '''
        get the challenge time is in the specified range

        :return: the defined validation timeout in seconds
        :rtype:  int
        '''
        try:
            timeout = int(getFromConfig("SMSProviderTimeout", 5 * 60))
        except Exception as ex:
            log.warning("SMSProviderTimeout: value error %r - reset to 5*60"
                                                                        % (ex))
            timeout = 5 * 60
        return timeout

###eof#########################################################################




'''
privacyIDEA is an open solution for strong two-factor authentication with One Time Passwords.

The core modules and basic necessary components are licensed under the AGPLv3, so that you 
are able to have a complete working open source solution. But privacyIDEA is also open as far 
as its modular architecture is concerned.   
privacyIDEA aims to not bind you to any  decision of the authentication protocol or it does not 
dictate you where your user information should be stored. This is achieved by its new, totally 
modular architecture. 

The modules are:

Tokenclasses
------------

    privacyIDEA already comes with several tokenclasses defined in privacyidea.lib.tokenclass.py
    But you can simply define your own tokenclass object. Take a look at the base class
    in tokenclass.py
    
UserIdResolvers
---------------
    
Controllers
===========
    privacyIDEA runs as a web application, so you may communitcate with privacyIDEA using 
    web calls.
    
Authentication Interface
------------------------
    user the controller /validate/....

Admin interface
---------------
    use the controller /admin/....
'''


# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
This file is part of the privacyidea service
Interface for the Challenge Response Token (OCRA)
'''

import traceback
import logging
from urllib import urlencode


from pylons import request, response, config, tmpl_context as c
from privacyidea.model.meta import Session

from privacyidea.lib.base import BaseController
from privacyidea.weblib.util import get_client
from privacyidea.lib.error   import ParameterError

from privacyidea.lib.util    import getParam, getLowerParams
from privacyidea.lib.reply   import sendResult, sendError
from privacyidea.lib.reply   import sendQRImageResult


from privacyidea.lib.realm   import getDefaultRealm
from privacyidea.lib.user    import getUserFromRequest
from privacyidea.lib.user    import getUserInfo
from privacyidea.lib.user    import User

from privacyidea.lib.policy  import PolicyClass
from privacyidea.lib.policy  import PolicyException

from privacyidea.lib.token import getTokens4UserOrSerial
from privacyidea.lib.tokenclass import OcraTokenClass
from privacyidea.lib.token import get_token_type_list
from privacyidea.lib.token import checkSerialPass
from privacyidea.lib.user import  getUserFromParam
from privacyidea.lib.config import get_privacyIDEA_config
from privacyidea.lib.log import log_with
import webob


log = logging.getLogger(__name__)

optional = True
required = False


class OcraController(BaseController):
    '''
    The OcraController implements challenges/response tokens according to RFC 6287
    '''

    @log_with(log)
    def __before__(self, action, **params):
        '''
        Here we see, what action is to be called and check the authorization
        '''

        try:
            c.audit['success'] = False
            c.audit['client'] = get_client()
            self.Policy = PolicyClass(request, config, c,
                                      get_privacyIDEA_config(),
                                      token_type_list = get_token_type_list())
            if action != "check_t":
                self.before_identity_check(action)

            return response

        except webob.exc.HTTPUnauthorized as acc:
            ## the exception, when an abort() is called if forwarded
            log.error("%r webob.exception %r" % (action, acc))
            log.error(traceback.format_exc())
            Session.rollback()
            Session.close()
            raise acc

        except Exception as exx:
            log.error("%r exception %r" % (action, exx))
            log.error(traceback.format_exc())
            Session.rollback()
            Session.close()
            return sendError(response, exx, context='before')

        finally:
            pass

    @log_with(log)
    def __after__(self, action, **params):
        c.audit['administrator'] = getUserFromRequest(request).get("login")
        self.audit.log(c.audit)

        return response


########################################################
    @log_with(log)
    def request(self, action, **params):
        """
        method:
            orcra/request

        description:
            request a challenge for a user or for a serial number (token).

        arguments:
            * serial: (required - string)
                    Serial number of the token, for which a challenge should
                    be generated (either serial or user is required)

            * user:   (required  - string)
                    The user for whose token a challenge should be generated
                    If the user has more than one token, an error is returend.
                    (either serial or user is required)

            * data:   (required - String: URLendoced)
                    These are the display data, that can be used to generate the challenge

        remark:
            the app will report a wrong qrcode, if the policy
             {'authentication' : qrtanurl=https://localhost }
            is not defined !!

        returns:

            A JSON respone::

                        {
                            "version": "privacyIDEA 2.4",
                            "jsonrpc": "2.0",
                            "result": {
                                "status": true,
                                "value": false,
                            },
                            "detail": {
                                    "transactionid" : TRANSAKTIONSID,
                                    "data" : DATAOBJECT,
                            }
                        }

            * transactionid:
                    This is the transaction ID, that is used later for
                    verifying the Return code /TAN.

            * data:
                    This is an object (URL) which can be used to generate a
                    QR-Coide to be displayed to the QRTAN App


        exception:

        """
        res = {}
        description = 'ocra/request: request a challenge for a given user or token (serial). You must either provide a parameter "user" or a parameter "serial".'
        dataobj = ""

        try:
            param = getLowerParams(request.params)

            self.Policy.checkPolicyPre('ocra', "request")

            serial = getParam(param, 'serial', optional)
            user = getUserFromParam(param, optional)

            if user.isEmpty() and serial is None:
                ## raise exception
                log.error("user or serial is required")
                raise ParameterError("Usage: %s" % description, id=77)

            message = getParam(param, 'data'  , optional)
            if message is None:
                message = ''

            ## ocra token
            tokens = getTokens4UserOrSerial(user, serial)

            if len(tokens) > 1 :
                error = ('More than one token found: unable to create challenge '
                        'for (u:%r,s:%r)!' % (user, serial))
                log.error(error)
                raise Exception(error)

            if len(tokens) == 0:
                error = ('No token found: unable to create challenge for'
                          ' (u:%r,s:%r)!' % (user, serial))
                log.error(error)
                raise Exception(error)

            ocra = tokens[0]
            (transId, challenge, res, url) = ocra.challenge(message)

            u = urlencode({'u':str(url.encode("utf-8"))})

            uInfo = {'tr': transId, 'ch' : challenge,
                      'me': str(message.encode("utf-8")), 'u': u[2:]}
            detail = {"transactionid"   : transId,
                      'challenge'       : challenge,
                      'message'         : str(message.encode("utf-8")),
                      'url'             : str(url.encode("utf-8")),
                     }

            ## create the app_url from the data'''
            dataobj = 'lseqr://req?%s' % (str(urlencode(uInfo)))

            ## append the signature to the url '''
            signature = {'si' : ocra.signData(dataobj)}
            uInfo['si'] = signature
            dataobj = '%s&%s' % (dataobj, str(urlencode(signature)))

            detail["data"] = dataobj

            c.audit['success'] = res
            #c.audit['info'] += "%s=%s, " % (k, value)

            Session.commit()
            qr = getParam(param, 'qr', optional)
            if qr is not None:
                param['alt'] = detail
                return sendQRImageResult(response, dataobj, param)
            else:
                return sendResult(response, res, 1, opt=detail)

        except PolicyException as pe:
            log.error("policy failed: %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe))

        except Exception as exx:
            log.error("failed: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(exx))

        finally:
            Session.close()



    ## https://privacyideaserver/ocra/check_t?transactionid=TRANSACTIONID&pass=TAN
    @log_with(log)
    def check_t(self, action, **params):
        """
        method:
            orcra/check_t

        description:
            verify the response of the ocra token

        arguments:
            * transactionid:  (required - string)
                    Dies ist eine Transaktions-ID, die bei der Challenge ausgegeben wurde.

            * pass:   (required - string)
                    die response, die der OCRA Token auf Grund der Challenge berechnet hat

        returns:

            A JSON response::

                {
                 "version": "privacyIDEA 2.4",
                 "jsonrpc": "2.0",
                 "result": {
                     "status": true,
                     "value": {
                         "failcount" : 3,
                         "result": false
                        }
                    },
                 "id": 0
                }

        exception:

        """
        res = {}
        description = 'ocra/check_t: validate a token request.'

        try:
            param = getLowerParams(request.params)

            #checkPolicyPre('ocra', "check_t" )

            passw = getParam(param, 'pass'  , optional)
            if passw is None:
                ## raise exception'''
                log.error("missing pass ")
                raise ParameterError("Usage: %s Missing parameter 'pass'." % description, id=77)

            transid = getParam(param, 'transactionid', optional)
            if transid is None:
                ## raise exception'''
                log.error("missing transactionid, user or serial number of token")
                raise ParameterError("Usage: %s Missing parameter 'transactionid'." % description, id=77)

            ## if we have a transaction, get serial from this challenge
            value = {}
            ocraChallenge = OcraTokenClass.getTransaction(transid)
            if ocraChallenge is not None:
                serial = ocraChallenge.tokenserial

                tokens = getTokens4UserOrSerial(serial=serial)
                if len(tokens) == 0 or len(tokens) > 1:
                    raise Exception('tokenmismatch for token serial: %s'
                                    % (unicode(serial)))

                theToken = tokens[0]
                tok = theToken.token
                desc = tok.get()
                realms = desc.get('privacyIDEA.RealmNames')
                if realms is None or len(realms) == 0:
                    realm = getDefaultRealm()
                elif len(realms) > 0:
                    realm = realms[0]

                userInfo = getUserInfo(tok.privacyIDEAUserid, tok.privacyIDEAIdResolver, tok.privacyIDEAIdResClass)
                user = User(login=userInfo.get('username'), realm=realm)

                (ok, opt) = checkSerialPass(serial, passw, user=user,
                                     options={'transactionid':transid})

                failcount = theToken.getFailCount()
                value['result'] = ok
                value['failcount'] = int(failcount)

            else:
                ## no challenge found for this transid
                value['result'] = False
                value['failure'] = 'No challenge for transaction %r found'\
                                    % transid

            c.audit['success'] = res
            #c.audit['info'] += "%s=%s, " % (k, value)

            Session.commit()
            return sendResult(response, value, 1)

        except Exception as e :
            log.error("failed: %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendResult(response, unicode(e), 0)

        finally:
            Session.close()


    '''
        https://privacyideaserver/ocra/checkstatus?transactionid=TRANSACTIONID
        https://privacyideaserver/ocra/checkstatus?serial=SERIENNUMMER
        https://privacyideaserver/ocra/checkstatus?user=BENUTZER
    '''
    @log_with(log)
    def checkstatus(self, action, **params):
        """
        method:
            orcra/checkstatus

        description:
            Methode zur assynchronen Ueberpruefungen eines Challenge Response Valiadation requests

        arguments:

            * transactionid:  (required one of  - string - (hex))
                    Dies ist eine Transaktions-ID, die bei der Challenge ausgegeben wurde.

            * serial: (required one of  - string)
                    die Serien Nummer des OCRA Token

            * user: (required one of  - string)
                    die Benutzer eines Tokens

            required is one of (user,serial,transactionid)

        returns:

            A JSON response::

                {
                 "version": "privacyIDEA 2.4",
                 "jsonrpc": "2.0",
                 "result": {
                     "status": true,
                     "value": [
                             {
                             "serial": SERIENNUMMER1,
                             "transactionid": TRANSACTIONID1,
                             "received_tan": true,
                             "valid_tan": true,
                             "failcount": 0
                             },
                             {
                             "serial": SERIENNUMMER1,
                             "transactionid": TRANSACTIONID2,
                             "received_tan": false,
                             "valid_tan": false,
                             "failcount": 0
                             },
                             {
                             "serial": SERIENNUMMER2,
                             "transactionid": TRANSACTIONID3,
                             "received_tan": true,
                             "valid_tan": false,
                             "failcount": 2
                             },
                         ]
                     },
                 "id": 0
                 }

        exception:

        """
        res = {}
        description = 'ocra/checkstatus: check the token status - for assynchronous verification. Missing parameter: You need to provide one of the parameters "transactionid", "user" or "serial"'

        try:
            param = getLowerParams(request.params)

            self.Policy.checkPolicyPre('ocra', "status")

            transid = getParam(param, 'transactionid'   , optional)
            user = getUserFromParam(param, optional)
            #user   = getParam(param, 'user'          ,optional)
            serial = getParam(param, 'serial'          , optional)

            if transid is None and user.isEmpty() and serial is None:
                ## raise exception
                log.error("missing transactionid, user or serial number for token")
                raise ParameterError("Usage: %s" % description, id=77)

            tokens = []
            serials = set()
            status = []

            if serial is not None:
                serials.add(serial)

            ## if we have a transaction, get serial from this challenge
            if transid is not None :
                ocraChallenge = None
                try:
                    ocraChallenge = OcraTokenClass.getTransaction(transid)
                except:
                    pass
                if ocraChallenge is not None:
                    serials.add(ocraChallenge.tokenserial)

            ## if we have a serial number of  token
            if len(serials) > 0:
                for serial in serials:
                    tokens.extend(getTokens4UserOrSerial(serial=serial))

            ## if we have a user
            if user.isEmpty() == False:
                try:
                    tokens.extend(getTokens4UserOrSerial(user=user))
                except:
                    log.warning("no token or user %r found!" % user)

            for token in tokens:
                if token.getType() == 'ocra':
                    challenges = []
                    if transid is None:
                        serial = token.getSerial()
                        challenges = OcraTokenClass.getTransactions4serial(serial)
                    else:
                        challenges.append(OcraTokenClass.getTransaction(transid))

                    for challenge in challenges:
                        stat = token.getStatus(challenge.transid)
                        if stat is not None and len(stat) > 0:
                            status.append(stat)

            res['values'] = status
            c.audit['success'] = res

            Session.commit()
            return sendResult(response, res, 1)

        except PolicyException as pe:
            log.error("policy failed: %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe))

        except Exception as exx:
            log.error("failed: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendResult(response, unicode(exx), 0)

        finally:
            Session.close()



    @log_with(log)
    def getActivationCode(self, action, **params):
        '''
        method:
            orcra/getActivationCode

        description:
            returns an valid example activcation code

        arguments:
            ./.

        returns:
            JSON with     "activationcode": "JZXW4ZI=2A"
        '''

        from privacyidea.lib.crypto import createActivationCode

        res = {}
        #description = 'ocra/getActivationCode'

        try:
            params = getLowerParams(request.params)

            self.Policy.checkPolicyPre('ocra', "activationcode")

            ac = str(params.get('activationcode'))
            activationCode = createActivationCode(acode=ac)
            res = {'activationcode':activationCode}

            Session.commit()
            return sendResult(response, res, 1)

        except PolicyException as pe:
            log.error("policy failed: %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe))

        except Exception as exx:
            log.error("failed: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(exx), 0)

        finally:
            Session.close()


    @log_with(log)
    def calculateOtp(self, action, **params):
        '''

        '''
        from privacyidea.lib.crypto import kdf2
        from privacyidea.lib.ocra import OcraSuite
        from datetime import datetime

        from urlparse import urlparse
        from urlparse import parse_qs

        res = {}
        #description = 'ocra/calculateOtp: calculate the first otp from the given init2 response '

        try:
            params = getLowerParams(request.params)

            self.Policy.checkPolicyPre('ocra', "calcOTP")

            sharedsecret = params.get('sharedsecret')
            activationcode = params.get('activationcode')
            nonce = params.get('nonce')
            ocrasuite = params.get('ocrasuite')
            challenge = params.get('challenge')
            counter = params.get('counter')
            ocrapin = params.get('ocrapin')

            nonce3 = params.get('no')
            ocrasuite3 = params.get('os')
            #serial3         = params.get('se')

            challenge = params.get('challenge')
            counter = params.get('counter')
            ocrapin = params.get('ocrapin')
            init1 = params.get('init1')
            init2 = params.get('init2')

            ## parse init1 '''
            if init1 is not None:
                ## now parse the appurl for the ocrasuite '''
                uri = urlparse(init1.replace('lseqr://', 'http://'))
                qs = uri.query
                qdict = parse_qs(qs)

                ocrasuite2 = qdict.get('os', None)
                if ocrasuite2 is not None and len(ocrasuite2) > 0:
                    ocrasuite2 = ocrasuite2[0]

                if ocrasuite is None:
                    ocrasuite = ocrasuite2

                sharedsecret2 = qdict.get('sh', None)
                if sharedsecret2 is not None and len(sharedsecret2) > 0:
                    sharedsecret2 = sharedsecret2[0]

                if sharedsecret is None:
                    sharedsecret = sharedsecret2

            ## parse init1
            if init2 is not None:
                ## now parse the appurl for the ocrasuite
                uri = urlparse(init2.replace('lseqr://', 'http://'))
                qs = uri.query
                qdict = parse_qs(qs)

                challenge2 = qdict.get('ch', None)
                if challenge2 is not None and len(challenge2) > 0:
                    challenge2 = challenge2[0]
                if challenge is None:
                    challenge = challenge2

                nonce2 = qdict.get('no', None)
                if nonce2 is not None and len(nonce2) > 0:
                    nonce2 = nonce2[0]
                if nonce is None:
                    nonce = nonce2

            if ocrapin is None:
                ocrapin = ''
            if counter is None:
                counter = 0

            if nonce3 is not None:
                nonce = unicode(nonce3)

            if ocrasuite3 is not None:
                ocrasuite = unicode(ocrasuite3)

            ##  now we have all in place for the key derivation to create the new key
            ##     sharedsecret, activationcode and nonce
            key_len = 20
            if ocrasuite.find('-SHA256'):
                key_len = 32
            elif ocrasuite.find('-SHA512'):
                key_len = 64

            if sharedsecret is not None:
                sharedsecret = unicode(sharedsecret)
            if nonce is not None:
                nonce = unicode(nonce)
            if activationcode is not None:
                activationcode = unicode(activationcode)

            newkey = kdf2(sharedsecret, nonce, activationcode, len=key_len)
            ## hnewkey = binascii.hexlify(newkey)
            ocra = OcraSuite(ocrasuite)

            param = {}
            param['C'] = int(counter)
            param['Q'] = unicode(challenge)
            param['P'] = unicode(ocrapin)
            param['S'] = ''
            if ocra.T is not None:
                ## Default value for G is 1M, i.e., time-step size is one minute and the
                ##  T represents the number of minutes since epoch time [UT].
                now = datetime.now()
                stime = now.strftime("%s")
                itime = int(stime)
                param['T'] = itime

            data = ocra.combineData(**param)
            otp = ocra.compute(data, newkey)

            res = {'otp':otp}

            Session.commit()
            return sendResult(response, res, 1)

        except PolicyException as pe:
            log.error("policy failed: %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, pe)

        except Exception as e:
            log.error("failed: %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(e), 0)

        finally:
            Session.close()


#eof###########################################################################




# The MIT License (MIT)
#
# Copyright (c) 2014 Fatih Erikli
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from pyparsing import (
    Literal, White, Word, alphanums, CharsNotIn, Forward, Group,
    Optional, OneOrMore, ZeroOrMore, pythonStyleComment)


class NginxParser(object):
    """
    A class that parses nginx configuration with pyparsing
    """

    # constants
    left_bracket = Literal("{").suppress()
    right_bracket = Literal("}").suppress()
    semicolon = Literal(";").suppress()
    space = White().suppress()
    key = Word(alphanums + "_/")
    value = CharsNotIn("{};,")
    location = CharsNotIn("{};,     ")

    # rules
    assignment = (key + Optional(space + value) + semicolon)
    block = Forward()

    block << Group(
        Group(key + Optional(space + location))
        + left_bracket
        + Group(ZeroOrMore(Group(assignment) | block))
        + right_bracket)

    script = OneOrMore(Group(assignment) | block).ignore(pythonStyleComment)

    def __init__(self, source):
        self.source = source

    def parse(self):
        """
        Returns the parsed tree.
        """
        return self.script.parseString(self.source)

    def as_list(self):
        """
        Returns the list of tree.
        """
        return self.parse().asList()


class NginxDumper(object):
    """
    A class that dumps nginx configuration from the provided tree.
    """
    def __init__(self, blocks, indentation=4):
        self.blocks = blocks
        self.indentation = indentation

    def __iter__(self, blocks=None, current_indent=0, spacer=' '):
        """
        Iterates the dumped nginx content.
        """
        blocks = blocks or self.blocks
        for key, values in blocks:
            if current_indent:
                yield spacer
            indentation = spacer * current_indent
            if isinstance(key, list):
                yield indentation + spacer.join(key) + ' {'
                for parameter in values:
                    if isinstance(parameter[0], list):
                        dumped = self.__iter__([parameter],
                                               current_indent +
                                               self.indentation)
                        for line in dumped:
                            yield line
                    else:
                        dumped = spacer.join(parameter) + ';'
                        yield spacer * (current_indent + self.indentation) + dumped

                yield indentation + '}'
            else:
                yield spacer * current_indent + key + spacer + values + ';'

    def as_string(self):
        return '\n'.join(self)

    def to_file(self, out):
        for line in self:
            out.write(line)
        out.close()
        return out


# Shortcut functions to respect Python's serialization interface
# (like pyyaml, picker or json)

def loads(source):
    return NginxParser(source).as_list()


def load(_file):
    return loads(_file.read())


def dumps(blocks, indentation=4):
    return NginxDumper(blocks, indentation).as_string()


def dump(blocks, _file, indentation=4):
    return NginxDumper(blocks, indentation).to_file(_file)



# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216423.786438
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/templates/selfservice/login.mako'
_template_uri = '/selfservice/login.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = []


def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        c = context.get('c', UNDEFINED)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">\n<html>\n<head>\n<title>privacyIDEA</title>\n<meta name="author" content="Cornelius K\xf6lbel">\n<meta name="date" content="2010-07-05T23:23:25+0200">\n<meta name="keywords" content="privacyIDEA login">\n<meta http-equiv="content-type" content="text/html; charset=UTF-8">\n<meta http-equiv="content-type" content="application/xhtml+xml; charset=UTF-8">\n<meta http-equiv="content-style-type" content="text/css">\n\n<link type="text/css" rel="stylesheet" href="/selfservice/style.css" />\n<link type="text/css" rel="stylesheet" href="/selfservice/custom-style.css" />\n<script type="text/javascript" src="/js/jquery-1.7.2.min.js"></script>\n\n</head>\n\n<body>\n\n<script>\n$(document).ready(function() {\n    $(\'form:first *:input[type!=hidden]:first\').focus();\n   \t$(\'#wrongCredentials\').delay(2000).fadeOut("slow");\n});\n</script>\n\n<div id="wrap">\n\n<div id="header-login">\n\t<div id="logo">\n\t</div>\n\t\n\t<div class="float_right">\n\t<span class=portalname>')
        # SOURCE LINE 35
        __M_writer(escape(_("privacyIDEA")))
        __M_writer(u'</span>\n\t</div>\n</div>\n\n\n\n<div id="main-login">\n<h1>')
        # SOURCE LINE 42
        __M_writer(escape(_("Login to privacyIDEA")))
        __M_writer(u'\n</h1>\n\n  <p>\n    <form action="/account/dologin" method="POST">\n      <table>\n        <tr><td><label for=login>')
        # SOURCE LINE 48
        __M_writer(escape(_("Username")))
        __M_writer(u':</label></td>\n        <td><input type="text" id="login" name="login" value="" /></td></tr>\n')
        # SOURCE LINE 50
        if c.realmbox:
            # SOURCE LINE 51
            __M_writer(u'        \t<tr>\n')
            # SOURCE LINE 52
        else:
            # SOURCE LINE 53
            __M_writer(u'\t\t\t<tr style="display:none;">\n')
        # SOURCE LINE 55
        __M_writer(u'\t\t<td>')
        __M_writer(escape(_("Realm")))
        __M_writer(u':</td>\n        <td>\n\t    <select name="realm">\n')
        # SOURCE LINE 58
        for realm in c.realmArray:
            # SOURCE LINE 59
            if c.defaultRealm == realm:
                # SOURCE LINE 60
                __M_writer(u'\t        <option value="')
                __M_writer(escape(realm))
                __M_writer(u'" selected>')
                __M_writer(escape(realm))
                __M_writer(u'</option>\n')
                # SOURCE LINE 61
            else:
                # SOURCE LINE 62
                __M_writer(u'\t        <option value="')
                __M_writer(escape(realm))
                __M_writer(u'">')
                __M_writer(escape(realm))
                __M_writer(u'</option>\n')
        # SOURCE LINE 65
        __M_writer(u'        </select>\n        </td></tr>\n        <tr><td><label for=password>')
        # SOURCE LINE 67
        __M_writer(escape(_("Password")))
        __M_writer(u':</label></td>\n        <td>\n        <input autocomplete="off" type="password" id="password" name="password" value ="" />\n')
        # SOURCE LINE 70
        if c.login_help:
            # SOURCE LINE 71
            __M_writer(u"\t\t\t<a href='")
            __M_writer(escape(c.help_url))
            __M_writer(u'/webui/login.html\' target="_blank">\n\t\t\t<img alt="(?)" width=24\n\t\t\tsrc="/images/help32.png"  \n\t\t\ttitle=\'')
            # SOURCE LINE 74
            __M_writer(escape(_("Open help on WebUI Login")))
            __M_writer(u"'>\n\t\t\t</a>\n")
        # SOURCE LINE 77
        __M_writer(u'        </td></tr>\n        <tr><td></td><td class=passwordWarning id=wrongCredentials>')
        # SOURCE LINE 78
        __M_writer(escape(c.status))
        __M_writer(u'</td></tr>\n        <tr><td></td>\n        <td>   <input type="submit" value="Login" /></td></tr>\n      </table>\n    </form>\n  </p>\n\n<div id=\'errorDiv\'></div>\n<div id=\'successDiv\'></div>\n\n\n</div>  <!-- end of main-->\n\n<!--\n<div id="footer">\n\t')
        # SOURCE LINE 93
        __M_writer(escape(c.version))
        __M_writer(u' | ')
        __M_writer(escape(c.licenseinfo))
        __M_writer(u'\n</div>\n-->\n</div>  <!-- end of wrap -->\n</body>\n</html>\n\n\n\n\n\n')
        return ''
    finally:
        context.caller_stack._pop_frame()





# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''     
  Description:  functional tests
                
  Dependencies: -

'''

from privacyidea.tests import TestController, url

import json

import logging
log = logging.getLogger(__name__)


import threading

class DoRequest(threading.Thread):
    ''' the request thread'''

    def __init__ (self, utest, rid=1, uri=None, params=None):
        '''
        initialize all settings of the request thread

        :param utest: method/function to be called
        :param rid: the request id
        :param uri: the request url object
        :param params: additional parmeters
        '''
        threading.Thread.__init__(self)

        self.utest = utest
        self.rid = rid
        self.uri = uri
        self.params = params

        self.response = None

    def run(self):
        ''' start the thread'''
        response = self.utest.app.get(self.uri, params=self.params)
        self.response = response.body
        return

    def status(self):
        '''
        retrieve the request result

        :return: the thread request result
        '''
        res = '"status": true,' in self.response
        return res

    def stat(self):
        '''
        retrieve the complete response
        '''
        return (self.rid, self.response)

class TestFixesController(TestController):
    '''
    test some fixes for closed tickets
    '''
    def setUp(self):
        ''' setup the Test Controller'''
        TestController.setUp(self)
        self.serials = []

    def tearDown(self):
        ''' make the dishes'''
        self.remove_tokens()
        TestController.tearDown(self)
        return

    def remove_tokens(self):
        '''
        remove all tokens, which are in the internal array of serial

        :return: - nothing -
        '''
        for serial in self.serials:
            self.del_token(serial)
        return

    def del_token(self, serial):
        '''
        delet a token identified by his serial number

        :param serial: the token serial
        :return: the response object of admin/remove
        '''
        param = {"serial" : serial }
        response = self.app.get(url(controller='admin', action='remove'),
                                                            params=param)
        return response

    def get_config(self):
        '''
        get the privacyidea config

        :return: the response object of the system/getConfig
        '''
        param = {}
        response = self.app.get(url(controller='system', action='getConfig'),
                                                                params=param)
        return response


    def add_token(self, user, pin=None, serial=None, typ=None, key=None,
                        timeStep=60, timeShift=0, hashlib='sha1', otplen=8):
        '''
        add a token to privacyIDEA

        :param user: user that owns the token
        :param pin: token pin - if none use the user name
        :param serial: give serial number
        :parm typ: token type
        :param key: the secret key
        :param timeStep: the TOTP time step
        :param timeShift: the TOTP time shift
        :param hashlib: the hmac hashlib
        :param otplen: the otp length

        :return: tuple of serial and response
        '''
        if serial is None:
            serial = 's' + user

        if pin is None:
            pin = user

        if typ is None:
            typ = 'totp'

        param = { 'user': user, 'pin':pin, 'serial': serial, 'type':typ,
                 'timeStep':timeStep, 'otplen' : otplen, 'hashlib':hashlib}
        if timeShift != 0:
            param['timeShift'] = timeShift

        if key is not None:
            param['otpkey'] = key

        response = self.app.get(url(controller='admin', action='init'),
                                                                params=param)
        assert '"status": true,' in response

        return (serial, response)


    def test_ticket_425(self):
        '''
        Test #2425: test if setConfig is timing save

        1. run multiple setConfig threads concurrently
        2. verify that only one thread has written his config
        3. verify, that all config entries of this thread are in place

        config entries are of format: key_entryId = val_threadId
             eg. key_101 = val_4 belongs to thread 4 and is entry 101
        '''

        check_results = []
        numthreads = 20
        numkeys = 200

        params = {}
        for tid in range(numthreads):
            param = {}
            for kid in range(numkeys):
                key = 'key_%d' % (kid)
                val = 'val_%d' % (tid)
                param[key] = val
            params[tid] = param

        uri = url(controller='system', action='setConfig')

        for tid in range(numthreads):
            param = params.get(tid)
            current = DoRequest(self, rid=tid, uri=uri, params=param)
            check_results.append(current)
            current.start()

        ## wait till all threads are completed
        for req in check_results:
            req.join()

        ## now check in the config if all keys are there
        config = self.get_config()
        conf = json.loads(config.body)
        conf = conf.get('result').get('value')

        ## check for the keys and the values in the dict
        counter = 0
        valdict = set()
        for cconf in conf:
            if cconf.startswith('key_'):
                valdict.add(conf.get(cconf))
                counter += 1

        assert counter == numkeys
        assert len(valdict) == 1

        return


    def test_ticket_864(self):
        '''
        #2864: admin/tokenrealm with multiple realms
        remarks:
            the problem is independent of sqlite, the reason is that realms are
            treated case insensitive
        1. create a token
        2. add some realms to the token
        3. verify, that the token is part of the realms
        '''

        sqlconnect = self.appconf.get('sqlalchemy.url')
        log.debug('current test against %s' % (sqlconnect))

        self.add_token('root', serial='troot', typ='spass', key='1234')

        param = {'serial':'troot', 'realms':'myDefRealm,myMixRealm'}
        response = self.app.get(url(controller='admin', action='tokenrealm'),
                                                                params=param)
        if '"value": 1' not in response.body:
            assert '"value": 1' in response.body

        param = {}
        ## the admin show returns slices of 10 token and our troot is not in
        ## the first slice :-( - so we now search directly for the token
        param['serial'] = 'troot'
        response = self.app.get(url(controller='admin', action='show'),
                                                            params=param)
        resp = json.loads(response.body)
        tok_data = resp.get('result').get('value').get('data')[0]
        realms = tok_data.get('privacyIDEA.RealmNames')
        t_ser = tok_data.get("privacyIDEA.TokenSerialnumber")

        assert t_ser == 'troot'
        assert 'mydefrealm' in realms
        assert 'mymixrealm' in realms

        self.del_token('troot')

        return

    def test_ticket_2909(self):
        '''
        Test #2909: HSM problems will raise an HSM Exception
               which could trigger an HTTP Error
        '''
        param = {'__HSMEXCEPTION__':'__ON__'}
        response = self.app.get(url(controller='system',
                                action='setupSecurityModule'), params=param)

        param = {'key':'sec', 'value':'mySec', 'type':'password'}
        response = self.app.get(url(controller='system', action='setConfig'),
                                                                params=param)

        assert '707' in response
        assert 'hsm not ready' in response

        res = ''
        try:
            param = {'key':'sec', 'value':'mySec',
                 'type':'password', 'httperror':'503'}
            response = self.app.get(url(controller='system',
                                        action='setConfig'), params=param)
        except Exception as exx:
            log.info(response)
            res = type(exx).__name__

        assert res == 'AppError'

        ## restore default
        param = {'__HSMEXCEPTION__':'__OFF__'}
        response = self.app.get(url(controller='system',
                                action='setupSecurityModule'), params=param)

        return

    def test_ticket_12018(self):
        '''
        #12018: OTPLen of /admin/init is not ignored
        '''
        (serial, response) = self.add_token('root', serial='troot',
                                           typ='hmac', key='1234', otplen=12)
        log.info(response)
        assert serial == 'troot'

        param = {}
        response = self.app.get(url(controller='admin', action='show'),
                                                                params=param)
        #resp = json.loads(response.body)
        assert '"privacyIDEA.OtpLen": 12' in response

        res = self.del_token(serial)
        assert '"status": true,' in res
        assert '"value": 1' in res

        return

#eof###########################################################################




# -*- coding: utf-8 -*-
#
#  May 08, 2014 Author: Cornelius Kölbel
#  contact:  http://www.privacyidea.org
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
from privacyidea.lib.policy import PolicyClass
from privacyidea.lib.config import get_privacyIDEA_config
from privacyidea.lib.log import log_with
from pylons import config as ini_config
import urllib
import httplib2
import json
import crypt
ENCODING = "utf-8"
import logging
from pylons import request, config, tmpl_context as c


log = logging.getLogger(__name__)


@log_with(log)
def _get_superuser_realms():
    '''
    return the list of realms, that contain superusers.
    '''
    superuser_realms = ["admin"]
    ini_su = ini_config.get("privacyideaSuperuserRealms", "")
    ini_su_list = [w.strip() for w in ini_su.split(",")]
    for r in ini_su_list:
        superuser_realms.append(r.lower())
    return superuser_realms


@log_with(log)
def check_admin_password(user, password, realm="admin"):
    '''
    Check the password of the admin user from the file defined in
    ini: privacyideaSuperuserFile
    
    :param user: the username used to login
    :param password: the password provided at login
    :return: The identity (user@realm) of the user or None
    :rtype: string
    '''
    success = None
    
    # check in the admin file
    admin_file = ini_config.get("privacyideaSuperuserFile")
    if admin_file:
        fileHandle = open(admin_file, "r")
        line = fileHandle.readline()
        while line:
            line = line.strip()
            fields = line.split(":", 7)
            if fields[0] == user:
                crypted_pass = fields[1]
                if crypt.crypt(password, crypted_pass) == crypted_pass:
                    success = "%s@%s" % (user, realm)
                    break

            line = fileHandle.readline()
        fileHandle.close()
        
    # check in LinOTP
    if not success:
        success = authenticate_privacyidea_user(user, realm, password)
                
    return success


@log_with(log)
def authenticate_privacyidea_user(user, realm, password):
    '''
    this function performs an authentication against the
    privacyidea server.
    
    :param user: Username of the user
    :type user: string
    :return: In case of success return the username
    :rtype: string
    '''
    res = False
    success = None
    Policy = PolicyClass(request,
                         config,
                         c,
                         get_privacyIDEA_config())
    if Policy.check_user_authorization(user, realm, exception=False):
        '''
        We SHOULD do it this way, but unfortunately we
        only get the complete context in a web request.
        We are missing the client and the HSM!
         
        (res, _opt) = checkUserPass(User(login=user, realm=realm), password)
        
        Big FIXME: The server is asking himself... :-/
        '''
        # we need to pass the client= to cope with client dependent policies.
        # Otherwise the authentication request will have the client 127.0.0.1
        # as the source.
        client = request.client_addr
        data = urllib.urlencode({'user': user,
                                 'realm': realm,
                                 'pass': password,
                                 'client': client})
        url = ini_config.get("privacyideaURL") + "/validate/check"
        disable_ssl = ini_config.get("privacyideaURL.disable_ssl", False)
        headers = {"Content-type": "application/x-www-form-urlencoded",
                   "Accept": "text/plain"}
        try:
            # is httplib compiled with ssl?
            http = httplib2.\
                Http(disable_ssl_certificate_validation=disable_ssl)
        except TypeError as exx:
            # not so on squeeze:
            # TypeError: __init__() got an unexpected keyword argument
            # 'disable_ssl_certificate_validation'
            log.warning("httplib2 'disable_ssl_certificate_validation' "
                        "attribute error: %r" % exx)
            # so we run in fallback mode
            http = httplib2.Http()
        (_resp, content) = http.request(url,
                                        method="POST",
                                        body=data,
                                        headers=headers)
        rv = json.loads(content)
        if rv.get("result"):
            # in case of normal json output
            res = rv['result'].get('value', False)

        if res:
            success = "%s@%s" % (user, realm)

    return success


@log_with(log)
def is_admin_identity(identity, exception=True):
    '''
    Check if the repoze identity is an admin, who is allowed to
    use the /admin, /system and all management controllers
    
    This is checked by verifying the standard realmname "admin" and
    the realms defined in privacyideaSuperuserRealms.
    
    Fixme: We need to get rid of the standard realm
    
    :param exception: If set to False the method will return a bool
                      value otherwise it will throw an exception
    :type exception: bool
    '''
    # During selftest the identity can be None!
    if identity is None:
        return False
    
    res = True
    try:
        # the identity could be a repoze identity object
        user_id = identity.get('repoze.who.userid')
        if type(user_id) == unicode:
            user_id = user_id.encode(ENCODING)
        identity = user_id.decode(ENCODING)
    except:
        pass
        
    (user, _foo, realm) = identity.rpartition('@')
    if realm.lower() not in _get_superuser_realms():
        if exception:
            path = request.path.lower()
            log.warning("User %s@%s tried to call the admin function %s."
                        % (user, realm, path))
            raise Exception("You are not an admin user and are not allowed "
                            "to call this function!")
        else:
            res = False
    return res



# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  2014-10-17 Fix the audit search with trailing /
#             Cornelius Kölbel <cornelius@privacyidea.org>
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''This file is part of the privacyidea service
'''


import logging


from pylons import tmpl_context as c
from pylons import request, response, config
from privacyidea.lib.base import BaseController

from privacyidea.lib.user import getUserFromRequest
from privacyidea.lib.policy import PolicyClass, PolicyException

from privacyidea.lib.reply import sendError
from privacyidea.lib.audit import CSVAuditIterator
from privacyidea.lib.audit import JSONAuditIterator
from privacyidea.lib.token import get_token_type_list
from privacyidea.lib.util import getParam
from privacyidea.weblib.util import get_client
from privacyidea.lib.config import get_privacyIDEA_config
from privacyidea.model.meta import Session
from privacyidea.lib.log import log_with

ENCODING = "utf-8"
import traceback


optional = True
required = False

log = logging.getLogger(__name__)


class AuditController(BaseController):

    '''
    this is the controller for doing some audit stuff

        https://server/audit/<functionname>

    '''

    @log_with(log)
    def __before__(self, action, **params):

        try:
            c.audit['client'] = get_client()
            self.before_identity_check(action)
            
        except Exception as exx:
            log.error("%r exception %r" % (action, exx))
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx, context='before')

        finally:
            Session.close()
            
    @log_with(log)
    def __after__(self, action, **params):
        try:
            c.audit['administrator'] = getUserFromRequest(request).get("login")
            self.audit.log(c.audit)
        finally:
            Session.close()

    @log_with(log)
    def search(self, action, **params):

        '''
        This functions searches within the audit trail
        It returns the audit information for the given search pattern

        method:
            audit/search

        arguments:
            key, value pairs as search patterns.

            * outform - optional: if set to "csv", than the token list will be
                        given in CSV


            or: Usually the key=values will be locally AND concatenated.
                it a parameter or=true is passed, the filters will
                be OR concatenated.

            The Flexigrid provides us the following parameters:
                ('page', u'1'), ('rp', u'100'),
                ('sortname', u'number'),
                ('sortorder', u'asc'),
                ('query', u''), ('qtype', u'serial')]
        returns:
            JSON response or csv format
        '''

        param = {}
        try:
            param.update(request.params)

            output_format = getParam(param, "outform", optional)
            Policy = PolicyClass(request, config, c,
                                 get_privacyIDEA_config(),
                                 token_type_list=get_token_type_list())
            Policy.checkPolicyPre('audit', 'view', {})

            # remove the param outform (and other parameters that should not
            # be used for search!
            search_params = {}
            for p in param:
                if p not in ["outform"]:
                    search_params[p] = param[p]

            log.debug("search params %r" % search_params)

            audit_iter = None

            if output_format == "csv":
                filename = "privacyidea-audit.csv"
                response.content_type = "application/force-download"
                response.headers['Content-'
                                 'disposition'] = ('attachment; filename=%s'
                                                   % filename)
                audit_iter = CSVAuditIterator(search_params)
            else:
                response.content_type = 'application/json'
                audit_iter = JSONAuditIterator(search_params)

            c.audit['success'] = True
            Session.commit()
            return audit_iter

        except PolicyException as pe:
            log.error("gettoken/getotp policy failed: %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:
            log.error("audit/search failed: %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response,
                             "audit/search failed: %s" % unicode(e), 0)

        finally:
            Session.close()


# eof########################################################################
